import type { BaseStyleProps } from '@antv/g';
import type { ID } from '@antv/graphlib';
import type { RuntimeContext } from '../runtime/types';
import type { EdgeDirection, Element, IDragEvent, Point, PrefixObject } from '../types';
import type { BaseBehaviorOptions } from './base-behavior';
import { BaseBehavior } from './base-behavior';
export interface DragElementOptions extends BaseBehaviorOptions, PrefixObject<BaseStyleProps, 'shadow'> {
    /**
     * <zh/> 是否启用拖拽动画
     *
     * <en/> Whether to enable drag animation
     */
    animation: boolean;
    /**
     * <zh/> 是否启用拖拽节点的功能
     *
     * <en/> Whether to enable the function of dragging the node
     */
    enable?: boolean | ((event: IElementDragEvent) => boolean);
    /**
     * <zh/> 拖拽操作效果
     * - link: 将拖拽元素置入为目标元素的子元素
     * - move: 移动元素并更新父元素尺寸
     * - none: 仅更新拖拽目标位置，不做任何额外操作
     *
     * <en/> Drag operation effect
     * - link: Place the drag element as a child element of the target element
     * - move: Move the element and update the parent element size
     * - none: Only update the drag target position, no additional operations
     * @description
     * <zh/> combo 元素可作为元素容器置入 node 或 combo 元素
     *
     * <en/> The combo element can be placed as an element container into the node or combo element
     */
    dropEffect: 'link' | 'move' | 'none';
    /**
     * <zh/> 节点选中的状态，启用多选时会基于该状态查找选中的节点
     *
     * <en/> The state name of the selected node, when multi-selection is enabled, the selected nodes will be found based on this state
     */
    state?: string;
    /**
     * <zh/> 拖拽时隐藏的边
     * - none: 不隐藏
     * - out: 隐藏以节点为源节点的边
     * - in: 隐藏以节点为目标节点的边
     * - both: 隐藏与节点相关的所有边
     * - all: 隐藏图中所有边
     *
     * <en/> Edges hidden during dragging
     * - none: do not hide
     * - out: hide the edges with the node as the source node
     * - in: hide the edges with the node as the target node
     * - both: hide all edges related to the node
     * - all: hide all edges in the graph
     * @description
     * <zh/> 使用幽灵节点时不会隐藏边
     *
     * <en/> Edges will not be hidden when using the drag shadow
     */
    hideEdge?: 'none' | 'all' | EdgeDirection;
    /**
     * <zh/> 是否启用幽灵节点，即用一个图形代替节点跟随鼠标移动
     *
     * <en/> Whether to enable the drag shadow, that is, use a shape to replace the node to follow the mouse movement
     */
    shadow?: boolean;
    /**
     * <zh/> 完成拖拽时的回调
     *
     * <en/> Callback when dragging is completed
     */
    onFinish?: (ids: ID[]) => void;
}
export declare class DragElement extends BaseBehavior<DragElementOptions> {
    static defaultOptions: Partial<DragElementOptions>;
    protected enable: boolean;
    private enableElements;
    protected target: ID[];
    private shadow?;
    private shadowOrigin;
    private hiddenEdges;
    private get animation();
    constructor(context: RuntimeContext, options: DragElementOptions);
    update(options: Partial<DragElementOptions>): void;
    private bindEvents;
    protected getSelectedNodeIDs(currTarget: ID[]): ID[];
    protected getDelta(event: IElementDragEvent): [number, number] | Float32Array | [number, number, number];
    protected onDragStart(event: IElementDragEvent): void;
    protected onDrag(event: IElementDragEvent): void;
    protected onDragEnd(): void;
    private onDrop;
    protected validate(event: IElementDragEvent): boolean;
    protected moveElement(ids: ID[], offset: Point): Promise<void>;
    private moveShadow;
    private createShadow;
    private showEdges;
    protected hideEdge(): void;
    private unbindEvents;
    destroy(): void;
}
type IElementDragEvent = IDragEvent<Element>;
export {};
