import type { AABB, DisplayObject, DisplayObjectConfig, Group } from '@antv/g';
import type { BaseElementStyleProps, Combo, Node, NodeLikeData, Padding, Placement, Position, PrefixObject, STDSize, Size } from '../../types';
import type { BaseNodeStyleProps } from '../nodes';
import { BaseNode } from '../nodes';
import { IconStyleProps } from '../shapes';
export type BaseComboStyleProps<T extends Record<string, unknown> = Record<string, any>> = T & BaseElementStyleProps & BaseNodeStyleProps<T> & PrefixObject<T, 'collapsed'> & PrefixObject<CollapsedMarkerStyleProps, 'collapsedMarker'> & {
    /**
     * <zh/> Combo 展开后的默认大小
     *
     * <en/> The default size of combo when expanded
     */
    size?: Size;
    /**
     * <zh/> Combo 收起后的默认大小
     *
     * <en/> The default size of combo when collapsed
     */
    collapsedSize?: Size;
    /**
     * <zh/> Combo 收起时的原点
     * <en/> The origin of combo when collapsed
     */
    collapsedOrigin?: Placement | [number, number];
    /**
     * <zh/> Combo 的子元素，可以是节点或者 Combo
     *
     * <en/> The children of combo, which can be nodes or combos
     */
    childrenNode?: (Node | Combo)[];
    /**
     * <zh/> Combo 的子元素数据
     *
     * <en/> The data of the children of combo
     * @description
     * <zh/> 如果 combo 是收起状态，children 可能为空，通过 childrenData 能够获取完整的子元素数据
     *
     * <en/> If the combo is collapsed, children may be empty, and the complete child element data can be obtained through childrenData
     */
    childrenData?: NodeLikeData[];
    /**
     * <zh/> Combo 的内边距，只在展开状态下生效
     * <en/> The padding of combo, only effective when expanded
     */
    padding?: Padding;
    collapsedMarker?: boolean;
};
export type CollapsedMarkerStyleProps = IconStyleProps & {
    /**
     * <zh/> 标记类型
     * <en/> Marker type
     *
     * - 'child-count': Number of child elements
     * - 'descendant-count': Number of descendant elements (including Nodes and Combos)
     * - 'node-count': Number of descendant elements (only Nodes)
     * - (children: NodeLikeData[]) => string: Custom function
     */
    type?: 'child-count' | 'descendant-count' | 'node-count' | ((children: NodeLikeData[]) => string);
};
export declare abstract class BaseCombo<S extends BaseComboStyleProps = BaseComboStyleProps> extends BaseNode<S> {
    type: string;
    static defaultStyleProps: Partial<BaseComboStyleProps>;
    constructor(options: DisplayObjectConfig<BaseComboStyleProps>);
    /**
     * Draw the key shape of combo
     */
    protected abstract drawKeyShape(attributes: Required<S>, container: Group): DisplayObject | undefined;
    protected calculatePosition(attributes: Required<S>): Position;
    protected getCollapsedOriginPosition(attributes: Required<S>): Position;
    protected getKeySize(attributes: Required<S>): STDSize;
    protected getEmptyKeySize(attributes: Required<S>): STDSize;
    protected getCollapsedKeySize(attributes: Required<S>): STDSize;
    protected getExpandedKeySize(attributes: Required<S>): STDSize;
    protected getContentBBox(attributes: Required<S>): AABB;
    protected drawCollapsedMarkerShape(attributes: Required<S>, container: Group): void;
    protected getCollapsedMarkerStyle(attributes: Required<S>): IconStyleProps | false;
    protected getCollapsedMarkerText(type: CollapsedMarkerStyleProps['type'], attributes: Required<S>): string;
    protected getComboZIndex(attributes: Required<S>): number;
    protected getComboStyle(attributes: Required<S>): {
        x: number;
        y: number;
        zIndex: number;
    };
    render(attributes: Required<S>, container?: Group): void;
    /**
     * <zh/> 在元素完成创建并执行完入场动画后调用
     *
     * <en/> Called after the element is created and the entrance animation is completed
     * @override
     */
    onCreate(): void;
    /**
     * <zh/> 在元素更新并执行完过渡动画后调用
     *
     * <en/> Called after the element is updated and the transition animation is completed
     * @override
     */
    onUpdate(): void;
    /**
     * <zh/> 在元素完成退场动画并销毁后调用
     *
     * <en/> Called after the element completes the exit animation and is destroyed
     * @override
     */
    onDestroy(): void;
}
