import type { DisplayObjectConfig, Group, ImageStyleProps, PathStyleProps } from '@antv/g';
import { Path } from '@antv/g';
import type { PathArray } from '@antv/util';
import type { BaseElementStyleProps, EdgeLabelStyleProps, Keyframe, LoopPlacement, Node, Point, PrefixObject, Size } from '../../types';
import type { SymbolFactor } from '../../utils/symbol';
import type { LabelStyleProps } from '../shapes';
import { BaseShape } from '../shapes/base-shape';
export type BaseEdgeStyleProps = BaseElementStyleProps & Pick<PathStyleProps, 'isBillboard' | 'markerStart' | 'markerStartOffset' | 'markerEnd' | 'markerEndOffset' | 'markerMid'> & PrefixObject<EdgeLabelStyleProps, 'label'> & PrefixObject<PathStyleProps, 'halo'> & PrefixObject<EdgeArrowStyleProps, 'startArrow'> & PrefixObject<EdgeArrowStyleProps, 'endArrow'> & PrefixObject<LoopStyleProps, 'loop'> & {
    /**
     * <zh/> 是否显示边的标签
     *
     * <en/> Whether to display the label of the edge
     */
    label?: boolean;
    /**
     * <zh/> 是否显示边的光晕
     *
     * <en/> Whether to display the halo of the edge
     */
    halo?: boolean;
    /**
     * <zh/> 是否显示边的起始箭头
     *
     * <en/> Whether to display the start arrow of the edge
     */
    startArrow?: boolean;
    /**
     * <zh/> 是否显示边的结束箭头
     *
     * <en/> Whether to display the end arrow of the edge
     */
    endArrow?: boolean;
    /**
     * <zh/> 起始箭头的偏移量
     *
     * <en/> Offset of the start arrow
     */
    startArrowOffset?: number;
    /**
     * <zh/> 结束箭头的偏移量
     *
     * <en/> Offset of the end arrow
     */
    endArrowOffset?: number;
    /**
     * <zh/> 主色
     * <en/> Subject color
     */
    color?: string;
    /**
     * <zh/> 边的起点 shape
     * <en/> The source shape. Represents the start of the edge
     */
    sourceNode: Node;
    /**
     * <zh/> 边的终点 shape
     * <en/> The source shape. Represents the start of the edge
     */
    targetNode: Node;
    /**
     * <zh/> 边起始连接的 port
     * <en/> The Port of the source node
     */
    sourcePort?: string;
    /**
     * <zh/> 边终点连接的 port
     * <en/> The Port of the target node
     */
    targetPort?: string;
};
type ParsedBaseEdgeStyleProps = Required<BaseEdgeStyleProps>;
export declare abstract class BaseEdge extends BaseShape<BaseEdgeStyleProps> {
    type: string;
    static defaultStyleProps: Partial<BaseEdgeStyleProps>;
    constructor(options: DisplayObjectConfig<BaseEdgeStyleProps>);
    protected getKeyStyle(attributes: ParsedBaseEdgeStyleProps): PathStyleProps;
    protected abstract getKeyPath(attributes: ParsedBaseEdgeStyleProps): PathArray;
    protected getLoopPath(attributes: ParsedBaseEdgeStyleProps): PathArray;
    protected getEndpoints(attributes: ParsedBaseEdgeStyleProps): [Point, Point];
    protected getHaloStyle(attributes: ParsedBaseEdgeStyleProps): false | PathStyleProps;
    protected getLabelStyle(attributes: ParsedBaseEdgeStyleProps): false | LabelStyleProps;
    protected drawArrow(attributes: ParsedBaseEdgeStyleProps, type: 'start' | 'end'): void;
    private getArrowStyle;
    protected drawLabelShape(attributes: ParsedBaseEdgeStyleProps, container: Group): void;
    protected drawHaloShape(attributes: ParsedBaseEdgeStyleProps, container: Group): void;
    protected drawKeyShape(attributes: ParsedBaseEdgeStyleProps, container: Group): Path | undefined;
    render(attributes?: Required<BaseEdgeStyleProps>, container?: Group): void;
    /**
     * <zh/> 在元素完成创建并执行完入场动画后调用
     *
     * <en/> Called after the element is created and the entrance animation is completed
     * @override
     */
    onCreate(): void;
    /**
     * <zh/> 在元素更新并执行完过渡动画后调用
     *
     * <en/> Called after the element is updated and the transition animation is completed
     * @override
     */
    onUpdate(): void;
    /**
     * <zh/> 在元素完成退场动画并销毁后调用
     *
     * <en/> Called after the element completes the exit animation and is destroyed
     * @override
     */
    onDestroy(): void;
    animate(keyframes: Keyframe[], options?: number | KeyframeAnimationOptions): import("@antv/g").IAnimation | null;
}
type SymbolName = 'triangle' | 'circle' | 'diamond' | 'vee' | 'rect' | 'triangleRect' | 'simple';
type EdgeArrowStyleProps = {
    type?: SymbolName | SymbolFactor;
    size?: Size;
} & PathStyleProps & Omit<ImageStyleProps, 'width' | 'height'> & Record<string, unknown>;
export type LoopStyleProps = {
    /**
     * <zh/> 边的位置
     * <en/> The position of the edge
     */
    placement?: LoopPlacement;
    /**
     * <zh/> 指定是否顺时针绘制环
     * <en/> Specify whether to draw the loop clockwise
     */
    clockwise?: boolean;
    /**
     * <zh/> 从节点 keyShape 边缘到自环顶部的距离，用于指定自环的曲率，默认为宽度或高度的最大值
     * <en/> Determine the position from the edge of the node keyShape to the top of the self-loop, used to specify the curvature of the self-loop, the default value is the maximum of the width or height
     */
    dist?: number;
};
export {};
