import EventEmitter from '@antv/event-emitter';
import type { AABB, BaseStyleProps } from '@antv/g';
import type { ID } from '@antv/graphlib';
import type { Plugin } from '../plugins/types';
import type { BehaviorOptions, ComboData, ComboOptions, EdgeData, EdgeOptions, G6Spec, GraphData, LayoutOptions, NodeData, NodeOptions, PluginOptions, ThemeOptions, TransformOptions } from '../spec';
import type { UpdateBehaviorOption } from '../spec/behavior';
import type { UpdatePluginOption } from '../spec/plugin';
import type { UpdateTransformOption } from '../spec/transform';
import type { CallableValue, DataID, EdgeDirection, ElementType, FitViewOptions, IEvent, NodeLikeData, PartialEdgeData, PartialGraphData, PartialNodeLikeData, Point, Position, State, ViewportAnimationEffectTiming } from '../types';
import type { DataURLOptions } from './canvas';
import { Canvas } from './canvas';
import type { HierarchyKey } from './data';
export declare class Graph extends EventEmitter {
    private options;
    static defaultOptions: G6Spec;
    destroyed: boolean;
    private context;
    constructor(options: G6Spec);
    /**
     * <zh/> 获取配置项
     *
     * <en/> Get options
     * @returns <zh/> 配置项 | <en/> options
     */
    getOptions(): G6Spec;
    /**
     * <zh/> 设置配置项
     *
     * <en/> Set options
     * @param options - <zh/> 配置项 | <en/> options
     * @description
     * <zh/> 要更新 devicePixelRatio、container 属性请销毁后重新创建实例
     *
     * <en/> To update devicePixelRatio and container properties, please destroy and recreate the instance
     */
    setOptions(options: G6Spec): void;
    setBackground(background: G6Spec['background']): void;
    getBackground(): G6Spec['background'];
    getSize(): [number, number];
    setSize(width: number, height: number): void;
    setZoomRange(zoomRange: G6Spec['zoomRange']): void;
    getZoomRange(): G6Spec['zoomRange'];
    setNode(node: NodeOptions): void;
    setEdge(edge: EdgeOptions): void;
    setCombo(combo: ComboOptions): void;
    getTheme(): ThemeOptions;
    setTheme(theme: CallableValue<ThemeOptions>): void;
    setLayout(layout: CallableValue<LayoutOptions>): void;
    getLayout(): LayoutOptions;
    /**
     * <zh/> 设置交互
     *
     * <en/> Set behaviors
     * @param behaviors - <zh/> 交互配置 | <en/> behavior configuration
     */
    setBehaviors(behaviors: CallableValue<BehaviorOptions>): void;
    /**
     * <zh/> 更新交互
     *
     * <en/> Update behavior
     * @param behavior - <zh/> 交互配置 | <en/> behavior configuration
     */
    updateBehavior(behavior: UpdateBehaviorOption): void;
    /**
     * <zh/> 获取交互
     *
     * <en/> Get behaviors
     * @returns <zh/> 交互配置 | <en/> behavior configuration
     */
    getBehaviors(): BehaviorOptions;
    /**
     * <zh/> 设置插件
     *
     * <en/> Set plugins
     * @param plugins - <zh/> 插件配置 | <en/> plugin configuration
     */
    setPlugins(plugins: CallableValue<PluginOptions>): void;
    /**
     * <zh/> 更新插件
     *
     * <en/> Update plugin
     * @param plugin - <zh/> 插件配置 | <en/> plugin configuration
     */
    updatePlugin(plugin: UpdatePluginOption): void;
    /**
     * <zh/> 获取插件
     *
     * <en/> Get plugins
     * @returns <zh/> 插件配置 | <en/> plugin configuration
     */
    getPlugins(): PluginOptions;
    /**
     * <zh/> 获取插件实例
     *
     * <en/> Get plugin instance
     * @param key - <zh/> 插件 key | <en/> plugin key
     * @returns <zh/> 插件实例 | <en/> plugin instance
     */
    getPluginInstance<T extends Plugin>(key: string): T;
    /**
     * <zh/> 设置数据转换器
     *
     * <en/> Set data transforms
     * @param transforms - <zh/> 数据转换配置 | <en/> data transform configuration
     */
    setTransforms(transforms: CallableValue<TransformOptions>): void;
    /**
     * <zh/> 更新数据转换器
     *
     * <en/> Update data transform
     * @param transform - <zh/> 数据转换配置 | <en/> data transform configuration
     */
    updateTransform(transform: UpdateTransformOption): void;
    /**
     * <zh/> 获取数据转换器
     *
     * <en/> Get data transforms
     * @returns <zh/> 数据转换器实例 | <en/> data transform instance
     */
    getTransforms(): TransformOptions;
    getData(): GraphData;
    getNodeData(): NodeData[];
    getNodeData(id: ID): NodeData;
    getNodeData(ids: ID[]): NodeData[];
    getEdgeData(): EdgeData[];
    getEdgeData(id: ID): EdgeData;
    getEdgeData(ids: ID[]): EdgeData[];
    getComboData(): ComboData[];
    getComboData(id: ID): ComboData;
    getComboData(ids: ID[]): ComboData[];
    setData(data: CallableValue<GraphData>): void;
    addData(data: CallableValue<GraphData>): void;
    addNodeData(data: CallableValue<NodeData[]>): void;
    addEdgeData(data: CallableValue<EdgeData[]>): void;
    addComboData(data: CallableValue<ComboData[]>): void;
    updateData(data: CallableValue<PartialGraphData>): void;
    updateNodeData(data: CallableValue<PartialNodeLikeData<NodeData>[]>): void;
    updateEdgeData(data: CallableValue<PartialEdgeData<EdgeData>[]>): void;
    updateComboData(data: CallableValue<PartialNodeLikeData<ComboData>[]>): void;
    removeData(id: DataID | ((data: GraphData) => DataID)): void;
    removeNodeData(ids: ID[] | ((data: NodeData[]) => ID[])): void;
    removeEdgeData(ids: ID[] | ((data: EdgeData[]) => ID[])): void;
    removeComboData(ids: ID[] | ((data: ComboData[]) => ID[])): void;
    getElementType(id: ID): ElementType;
    getRelatedEdgesData(id: ID, direction?: EdgeDirection): EdgeData[];
    getNeighborNodesData(id: ID): NodeData[];
    /**
     * <zh/> 获取节点或 combo 的祖先元素数据
     *
     * <en/> Get the ancestor element data of the node or combo
     * @param id - <zh/> 节点或 combo ID | <en/> node or combo ID
     * @param hierarchy - <zh/> 指定树图层级关系还是 combo 层级关系 | <en/> specify tree or combo hierarchy relationship
     * @returns <zh/> 祖先元素数据 | <en/> ancestor element data
     * @description
     * <zh/> 数组中的顺序是从父节点到祖先节点
     *
     * <en/> The order in the array is from the parent node to the ancestor node
     */
    getAncestorsData(id: ID, hierarchy: HierarchyKey): NodeLikeData[];
    /**
     * <zh/> 获取节点或 combo 的父元素数据
     *
     * <en/> Get the parent element data of the node or combo
     * @param id - <zh/> 节点或 combo ID | <en/> node or combo ID
     * @param hierarchy - <zh/> 指定树图层级关系还是 combo 层级关系 | <en/> specify tree or combo hierarchy relationship
     * @returns <zh/> 父元素数据 | <en/> parent element data
     */
    getParentData(id: ID, hierarchy: HierarchyKey): NodeLikeData | undefined;
    /**
     * <zh/> 获取节点或 combo 的子元素数据
     *
     * <en/> Get the child element data of the node or combo
     * @param id - <zh/> 节点或 combo ID | <en/> node or combo ID
     * @returns <zh/> 子元素数据 | <en/> child element data
     */
    getChildrenData(id: ID): NodeLikeData[];
    getElementDataByState(elementType: 'node', state: State): NodeData[];
    getElementDataByState(elementType: 'edge', state: State): EdgeData[];
    getElementDataByState(elementType: 'combo', state: State): ComboData[];
    private initCanvas;
    private createRuntime;
    private prepare;
    /**
     * <zh/> 执行渲染
     *
     * <en/> Render
     * @description
     * <zh/> 此过程会执行数据更新、绘制元素、执行布局
     *
     * <en/> This process will execute data update, element rendering, and layout execution
     */
    render(): Promise<void>;
    /**
     * <zh/> 绘制元素
     *
     * <en/> Draw elements
     * @returns <zh/> 渲染结果 | <en/> draw result
     */
    draw(): Promise<void>;
    layout(): Promise<void>;
    stopLayout(): void;
    /**
     * <zh/> 清空画布元素
     *
     * <en/> Clear canvas elements
     */
    clear(): Promise<void>;
    destroy(): void;
    getCanvas(): Canvas;
    resize(): void;
    resize(width: number, height: number): void;
    /**
     * <zh/> 将图缩放至合适大小并平移至视口中心
     *
     * <en/> Zoom the graph to fit the viewport and move it to the center of the viewport
     * @param options - <zh/> 适配配置 | <en/> fit options
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    fitView(options?: FitViewOptions, animation?: ViewportAnimationEffectTiming): Promise<void>;
    /**
     * <zh/> 将图平移至视口中心
     *
     * <en/> Move the graph to the center of the viewport
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    fitCenter(animation?: ViewportAnimationEffectTiming): Promise<void>;
    private autoFit;
    focusElement(id: ID | ID[], animation?: ViewportAnimationEffectTiming): Promise<void>;
    /**
     * <zh/> 基于当前缩放比例进行缩放
     *
     * <en/> Zoom based on the current zoom ratio
     * @param ratio - <zh/> 缩放比例 | <en/> zoom ratio
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     * @param origin - <zh/> 缩放中心(视口坐标) | <en/> zoom center(viewport coordinates)
     * @description
     * <zh/>
     * - ratio > 1 放大
     * - ratio < 1 缩小
     * <en/>
     * - ratio > 1 zoom in
     * - ratio < 1 zoom out
     */
    zoomBy(ratio: number, animation?: ViewportAnimationEffectTiming, origin?: Point): Promise<void>;
    /**
     * <zh/> 缩放画布至指定比例
     *
     * <en/> Zoom the canvas to the specified ratio
     * @param zoom - <zh/> 指定缩放比例 | <en/> specified zoom ratio
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     * @param origin - <zh/> 缩放中心(视口坐标) | <en/> zoom center(viewport coordinates)
     * @description
     * <zh/>
     * - zoom = 1 默认大小
     * - zoom > 1 放大
     * - zoom < 1 缩小
     * <en/>
     * - zoom = 1 default size
     * - zoom > 1 zoom in
     * - zoom < 1 zoom out
     */
    zoomTo(zoom: number, animation?: ViewportAnimationEffectTiming, origin?: Point): Promise<void>;
    /**
     * <zh/> 获取当前缩放比例
     *
     * <en/> Get the current zoom ratio
     * @returns <zh/> 缩放比例 | <en/> zoom ratio
     */
    getZoom(): number;
    /**
     * <zh/> 基于当前旋转角度进行旋转
     *
     * <en/> Rotate based on the current rotation angle
     * @param angle - <zh/> 旋转角度 | <en/> rotation angle
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     * @param origin - <zh/> 旋转中心(视口坐标) | <en/> rotation center(viewport coordinates)
     */
    rotateBy(angle: number, animation?: ViewportAnimationEffectTiming, origin?: Point): Promise<void>;
    /**
     * <zh/> 旋转画布至指定角度
     *
     * <en/> Rotate the canvas to the specified angle
     * @param angle - <zh/> 目标角度 | <en/> target angle
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     * @param origin - <zh/> 旋转中心(视口坐标) | <en/> rotation center(viewport coordinates)
     */
    rotateTo(angle: number, animation?: ViewportAnimationEffectTiming, origin?: Point): Promise<void>;
    /**
     * <zh/> 获取当前旋转角度
     *
     * <en/> Get the current rotation angle
     * @returns <zh/> 旋转角度 | <en/> rotation angle
     */
    getRotation(): number;
    /**
     * <zh/> 将图平移指定距离
     *
     * <en/> Translate the graph by the specified distance
     * @param offset - <zh/> 偏移量 | <en/> offset
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    translateBy(offset: Point, animation?: ViewportAnimationEffectTiming): Promise<void>;
    /**
     * <zh/> 将图平移至指定位置
     *
     * <en/> Translate the graph to the specified position
     * @param position - <zh/> 指定位置 | <en/> specified position
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     * @description
     */
    translateTo(position: Point, animation?: ViewportAnimationEffectTiming): Promise<void>;
    /**
     * <zh/> 获取图的位置
     *
     * <en/> Get the position of the graph
     * @returns <zh/> 图的位置 | <en/> position of the graph
     * @description
     * <zh/> 默认状态下，图的位置为 [0, 0]
     *
     * <en/> By default, the position of the graph is [0, 0]
     */
    getPosition(): Point;
    /**
     * <zh/> 将元素平移指定距离
     *
     * <en/> Translate the element by the specified distance
     * @param id - <zh/> 元素 ID | <en/> element ID
     * @param offset - <zh/> 偏移量 | <en/> offset
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    translateElementBy(id: ID, offset: Position, animation?: boolean): Promise<void>;
    /**
     * <zh/> 批量将元素平移指定距离
     *
     * <en/> Batch translate elements by the specified distance
     * @param offsets - <zh/> 偏移量配置 | <en/> offset configuration
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    translateElementBy(offsets: Record<ID, Position>, animation?: boolean): Promise<void>;
    /**
     * <zh/> 将元素平移至指定位置
     *
     * <en/> Translate the element to the specified position
     * @param id - <zh/> 元素 ID | <en/> element ID
     * @param position - <zh/> 指定位置 | <en/> specified position
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    translateElementTo(id: ID, position: Position, animation?: boolean): Promise<void>;
    /**
     * <zh/> 批量将元素平移至指定位置
     *
     * <en/> Batch translate elements to the specified position
     * @param positions - <zh/> 位置配置 | <en/> position configuration
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    translateElementTo(positions: Record<ID, Position>, animation?: boolean): Promise<void>;
    getElementPosition(id: ID): Position;
    getElementRenderStyle(id: ID): import("@antv/util/lib/lodash/types").ObjectType<unknown>;
    /**
     * <zh/> 设置元素可见性
     *
     * <en/> Set element visibility
     * @param id - <zh/> 元素 ID | <en/> element ID
     * @param visibility - <zh/> 可见性 | <en/> visibility
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    setElementVisibility(id: ID, visibility: BaseStyleProps['visibility'], animation?: boolean): Promise<void>;
    /**
     * <zh/> 批量设置元素可见性
     *
     * <en/> Batch set element visibility
     * @param visibility - <zh/> 可见性配置 | <en/> visibility configuration
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    setElementVisibility(visibility: Record<ID, BaseStyleProps['visibility']>, animation?: boolean): Promise<void>;
    /**
     * <zh/> 显示元素
     *
     * <en/> Show element
     * @param id - <zh/> 元素 ID | <en/> element ID
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    showElement(id: ID | ID[], animation?: boolean): Promise<void>;
    /**
     * <zh/> 隐藏元素
     *
     * <en/> Hide element
     * @param id - <zh/> 元素 ID | <en/> element ID
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    hideElement(id: ID | ID[], animation?: boolean): Promise<void>;
    /**
     * <zh/> 获取元素可见性
     *
     * <en/> Get element visibility
     * @param id - <zh/> 元素 ID | <en/> element ID
     * @returns <zh/> 元素可见性 | <en/> element visibility
     */
    getElementVisibility(id: ID): BaseStyleProps['visibility'];
    /**
     * <zh/> 设置元素层级
     *
     * <en/> Set element z-index
     * @param id - <zh/> 元素 ID | <en/> element ID
     * @param zIndex - <zh/> 层级 | <en/> z-index
     */
    setElementZIndex(id: ID, zIndex: number): Promise<void>;
    /**
     * <zh/> 批量设置元素层级
     *
     * <en/> Batch set element z-index
     * @param zIndex - <zh/> 层级配置 | <en/> z-index configuration
     */
    setElementZIndex(zIndex: Record<ID, number>): Promise<void>;
    /**
     * <zh/> 将元素置于最顶层
     *
     * <en/> Bring the element to the front
     * @param id - <zh/> 元素 ID | <en/> element ID
     */
    frontElement(id: ID | ID[]): Promise<void>;
    /**
     * <zh/> 获取元素层级
     *
     * <en/> Get element z-index
     * @param id - <zh/> 元素 ID | <en/> element ID
     * @returns <zh/> 元素层级 | <en/> element z-index
     */
    getElementZIndex(id: ID): number;
    /**
     * <zh/> 设置元素状态
     *
     * <en/> Set element state
     * @param id - <zh/> 元素 ID | <en/> element ID
     * @param state - <zh/> 状态 | <en/> state
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    setElementState(id: ID, state: State | State[], animation?: boolean): Promise<void>;
    /**
     * <zh/> 批量设置元素状态
     *
     * <en/> Batch set element state
     * @param state - <zh/> 状态配置 | <en/> state configuration
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    setElementState(state: Record<ID, State | State[]>, animation?: boolean): Promise<void>;
    /**
     * <zh/> 获取元素状态
     *
     * <en/> Get element state
     * @param id - <zh/> 元素 ID | <en/> element ID
     * @returns <zh/> 元素状态 | <en/> element state
     */
    getElementState(id: ID): State[];
    getElementRenderBounds(id: ID): AABB;
    collapse(id: ID, animation?: boolean): Promise<void>;
    expand(id: ID, animation?: boolean): Promise<void>;
    private setElementCollapsibility;
    /**
     * <zh/> 导出画布内容为 DataURL
     *
     * <en/> Export canvas content as DataURL
     * @param options - <zh/> 导出配置 | <en/> export configuration
     * @param mode
     * <zh/> 导出模式
     *  - viewport: 导出视口内容
     *  - overall: 导出整个画布
     *
     * <en/> export mode
     *  - viewport: export the content of the viewport
     *  - overall: export the entire canvas
     * @returns <zh/> DataURL | <en/> DataURL
     */
    toDataURL(options?: Partial<DataURLOptions>): Promise<string>;
    getCanvasByViewport(point: Point): Point;
    getViewportByCanvas(point: Point): Point;
    getClientByCanvas(point: Point): Point;
    getCanvasByClient(point: Point): Point;
    /**
     * <zh/> 获取视口中心的画布坐标
     *
     * <en/> Get the canvas coordinates of the viewport center
     * @returns <zh/> 视口中心的画布坐标 | <en/> Canvas coordinates of the viewport center
     */
    getViewportCenter(): Point;
    getCanvasCenter(): Point;
    private onResize;
    /**
     * <zh/> 监听事件
     *
     * <en/> Listen to events
     * @param eventName - <zh/> 事件名称 | <en/> event name
     * @param callback - <zh/> 回调函数 | <en/> callback function
     * @returns <zh/> Graph 实例 | <en/> Graph instance
     */
    on<T extends IEvent = IEvent>(eventName: string, callback: (event: T) => void): this;
    /**
     * <zh/> 一次性监听事件
     *
     * <en/> Listen to events once
     * @param eventName - <zh/> 事件名称 | <en/> event name
     * @param callback - <zh/> 回调函数 | <en/> callback function
     * @returns <zh/> Graph 实例 | <en/> Graph instance
     */
    once<T extends IEvent = IEvent>(eventName: string, callback: (event: T) => void): this;
}
