import { Rect } from '@antv/g';
import { isFunction } from '@antv/util';
import { CommonEvent } from '../constants';
import { getRectPoints, isBBoxCenterInRect } from '../utils/behaviors/brush';
import { getAllElementState, transformEdgeState } from '../utils/behaviors/utils';
import { Shortcut } from '../utils/shortcut';
import { BaseBehavior } from './base-behavior';
const SHOW_RECT_ID = 'g6-brush-select-rect-id';
export const DEFAULT_STYLE = {
    lineWidth: 1,
    color: '#EEF6FF',
    stroke: '#DDEEFE',
    fillOpacity: 0.4,
    zIndex: 2,
};
export class BrushSelect extends BaseBehavior {
    static defaultOptions = {
        enable: true,
        trigger: ['drag'],
        immediately: false,
        state: 'selected',
        mode: 'default',
        animation: false,
        enableElements: ['node', 'combo', 'edge'],
        style: {
            size: 0,
            type: 'rect',
            ...DEFAULT_STYLE,
            pointerEvents: 'none',
        },
    };
    startPoint;
    endPoint;
    rectShape;
    shortcut;
    selectElementFn = isBBoxCenterInRect;
    constructor(context, options) {
        super(context, Object.assign({}, BrushSelect.defaultOptions, options));
        this.shortcut = new Shortcut(context.graph);
        if (options.type === 'lasso-select')
            return;
        this.bindEvents();
    }
    pointerDown = async (event) => {
        if (!this.validate(event) || !this.isKeydown() || this.startPoint)
            return;
        const { style, trigger } = this.options;
        const triggers = (Array.isArray(trigger) ? trigger : [trigger]);
        if (event.targetType !== 'canvas' && triggers.includes('drag'))
            return;
        const { canvas } = this.context;
        this.rectShape = new Rect({
            id: SHOW_RECT_ID,
            style: {
                ...BrushSelect.defaultOptions.style,
                fill: style.color || DEFAULT_STYLE.color,
                ...style,
                pointerEvents: 'none',
            },
        });
        canvas.appendChild(this.rectShape);
        this.startPoint = [event.canvas.x, event.canvas.y];
    };
    pointerMove = async (event) => {
        if (!this.startPoint)
            return;
        const { immediately, mode } = this.options;
        this.endPoint = [event.canvas.x, event.canvas.y];
        this.rectShape?.attr({
            x: Math.min(this.endPoint[0], this.startPoint[0]),
            y: Math.min(this.endPoint[1], this.startPoint[1]),
            width: Math.abs(this.endPoint[0] - this.startPoint[0]),
            height: Math.abs(this.endPoint[1] - this.startPoint[1]),
        });
        if (immediately && mode === 'default') {
            this.updateElementState(getRectPoints(this.startPoint, this.endPoint));
        }
    };
    pointerUp = async (event) => {
        if (!this.startPoint)
            return;
        if (!this.endPoint) {
            await this.clearBrush();
            return;
        }
        const points = getRectPoints(this.startPoint, [event.canvas.x, event.canvas.y]);
        this.updateElementState(points);
        await this.clearBrush();
    };
    clearSelected = () => {
        if (this.endPoint)
            return;
        const { graph } = this.context;
        const selects = getAllElementState(graph, () => []);
        graph.setElementState(selects, this.options.animation);
    };
    updateElementState = (points) => {
        const { graph } = this.context;
        const { enableElements, state, mode, onSelect } = this.options;
        // 框选选中的 ids
        const rectSelectIds = this.getPointsSelectIds(graph, points, enableElements);
        // state mode 框选逻辑
        let stateChangeFn = (id, oldState) => (rectSelectIds.includes(id) ? [state] : []);
        switch (mode) {
            case 'union':
                stateChangeFn = (id, oldState) => (rectSelectIds.includes(id) ? [state] : oldState);
                break;
            case 'diff':
                stateChangeFn = (id, oldState) => {
                    if (rectSelectIds.includes(id)) {
                        return oldState.includes(state) ? [] : [state];
                    }
                    return oldState;
                };
                break;
            case 'intersect':
                stateChangeFn = (id, oldState) => {
                    if (rectSelectIds.includes(id)) {
                        return oldState.includes(state) ? [state] : [];
                    }
                    return oldState;
                };
                break;
        }
        let states = getAllElementState(graph, stateChangeFn);
        if (enableElements.includes('edge')) {
            transformEdgeState(graph, states, state);
        }
        if (isFunction(onSelect)) {
            states = onSelect(states);
        }
        graph.setElementState(states, this.options.animation);
    };
    clearBrush = async () => {
        this.rectShape?.remove();
        this.rectShape = undefined;
        this.startPoint = undefined;
        this.endPoint = undefined;
    };
    getPointsSelectIds = (graph, points, itemTypes) => {
        const selectedNodeIds = [];
        const selectedComboIds = [];
        if (itemTypes.includes('node')) {
            graph.getNodeData().forEach((node) => {
                const { id } = node;
                if (graph.getElementVisibility(id) !== 'hidden' && // hidden node is not selectable
                    this.selectElementFn(graph, id, points)) {
                    selectedNodeIds.push(id);
                }
            });
        }
        if (itemTypes.includes('combo')) {
            graph.getComboData().forEach((combo) => {
                const { id } = combo;
                if (graph.getElementVisibility(id) !== 'hidden' && // hidden combo is not selectable
                    this.selectElementFn(graph, id, points)) {
                    selectedComboIds.push(id);
                }
            });
        }
        return [...selectedNodeIds, ...selectedComboIds];
    };
    // 当前按键是否和 trigger 配置一致
    isKeydown() {
        const { trigger } = this.options;
        const keys = (Array.isArray(trigger) ? trigger : [trigger]);
        if (keys.length === 0 || keys.includes('drag'))
            return true;
        return this.shortcut?.match(keys);
    }
    validate(event) {
        if (this.destroyed)
            return false;
        const { enable } = this.options;
        if (isFunction(enable))
            return enable(event);
        return !!enable;
    }
    bindEvents() {
        const { graph } = this.context;
        this.unbindEvents();
        graph.on(CommonEvent.POINTER_DOWN, this.pointerDown);
        graph.on(CommonEvent.POINTER_MOVE, this.pointerMove);
        graph.on(CommonEvent.POINTER_UP, this.pointerUp);
        graph.on(`canvas:${CommonEvent.CLICK}`, this.clearSelected);
    }
    unbindEvents() {
        const { graph } = this.context;
        graph.off(CommonEvent.POINTER_DOWN, this.pointerDown);
        graph.off(CommonEvent.POINTER_MOVE, this.pointerMove);
        graph.off(CommonEvent.POINTER_UP, this.pointerUp);
        graph.off(`canvas:${CommonEvent.CLICK}`, this.clearSelected);
    }
    destroy() {
        this.unbindEvents();
        super.destroy();
    }
}
//# sourceMappingURL=brush-select.js.map