import { isFunction } from '@antv/util';
import { CommonEvent } from '../constants';
import { ELEMENT_TYPES } from '../constants/element';
import { idsOf } from '../utils/id';
import { getElementNthDegreeIds } from '../utils/relation';
import { Shortcut } from '../utils/shortcut';
import { BaseBehavior } from './base-behavior';
export class ClickElement extends BaseBehavior {
    selectedElementIds = [];
    shortcut;
    static defaultOptions = {
        animation: true,
        enable: true,
        multiple: false,
        trigger: ['shift'],
        selectedState: 'selected',
        unselectedState: undefined,
        degree: 0,
    };
    constructor(context, options) {
        super(context, Object.assign({}, ClickElement.defaultOptions, options));
        this.shortcut = new Shortcut(context.graph);
        this.bindEvents();
    }
    bindEvents() {
        const { graph } = this.context;
        this.unbindEvents();
        ELEMENT_TYPES.forEach((type) => {
            graph.on(`${type}:${CommonEvent.CLICK}`, this.onClickElement);
        });
        graph.on(`canvas:${CommonEvent.CLICK}`, this.onClickCanvas);
    }
    onClickElement = (event) => {
        if (!this.validate(event))
            return;
        this.updateElementsState(event, false);
        this.updateElementsState(event, true);
    };
    onClickCanvas = (event) => {
        if (!this.validate(event))
            return;
        this.updateElementsState(event, false);
        this.selectedElementIds = [];
    };
    updateElementsState = (event, add) => {
        if (!this.options.selectedState && !this.options.unselectedState)
            return;
        const { graph } = this.context;
        const { targetType, target } = event;
        if (add) {
            // 如果当前元素已经被选中，则取消选中 | If the current element is already selected, deselect it
            if (this.selectedElementIds.includes(target.id)) {
                this.selectedElementIds = this.selectedElementIds.filter((id) => id !== target.id);
            }
            else {
                const selectedElementIds = getElementNthDegreeIds(graph, targetType, target.id, this.options.degree);
                const isMultiple = this.options.multiple && this.shortcut.match(this.options.trigger);
                if (!isMultiple) {
                    this.selectedElementIds = selectedElementIds;
                }
                else {
                    this.selectedElementIds.push(...selectedElementIds);
                }
            }
        }
        if (!this.selectedElementIds.length)
            return;
        const states = {};
        if (this.options.selectedState) {
            Object.assign(states, this.getElementsState(this.selectedElementIds, this.options.selectedState, add));
        }
        if (this.options.unselectedState) {
            const inactiveIds = idsOf(graph.getData(), true).filter((id) => !this.selectedElementIds.includes(id));
            Object.assign(states, this.getElementsState(inactiveIds, this.options.unselectedState, add));
        }
        graph.setElementState(states, this.options.animation);
    };
    getElementsState = (ids, state, add) => {
        const { graph } = this.context;
        const states = {};
        ids.forEach((id) => {
            const currentState = graph.getElementState(id);
            const updatedState = add ? [...currentState, state] : currentState.filter((s) => s !== state);
            states[id] = updatedState;
        });
        return states;
    };
    validate(event) {
        if (this.destroyed)
            return false;
        const { enable } = this.options;
        if (isFunction(enable))
            return enable(event);
        return !!enable;
    }
    unbindEvents() {
        const { graph } = this.context;
        ELEMENT_TYPES.forEach((type) => {
            graph.off(`${type}:${CommonEvent.CLICK}`, this.onClickElement);
        });
        graph.off(`canvas:${CommonEvent.CLICK}`, this.onClickCanvas);
    }
    destroy() {
        this.unbindEvents();
        super.destroy();
    }
}
//# sourceMappingURL=click-element.js.map