import { Rect } from '@antv/g';
import { isFunction } from '@antv/util';
import { COMBO_KEY, CommonEvent } from '../constants';
import { getBBoxSize, getCombinedBBox } from '../utils/bbox';
import { idOf } from '../utils/id';
import { subStyleProps } from '../utils/prefix';
import { divide, subtract } from '../utils/vector';
import { BaseBehavior } from './base-behavior';
export class DragElement extends BaseBehavior {
    static defaultOptions = {
        animation: true,
        enable: (event) => ['node', 'combo'].includes(event.targetType),
        dropEffect: 'move',
        state: 'selected',
        hideEdge: 'none',
        shadowZIndex: 100,
        shadowFill: '#F3F9FF',
        shadowFillOpacity: 0.5,
        shadowStroke: '#1890FF',
        shadowStrokeOpacity: 0.9,
        shadowLineDash: [5, 5],
    };
    enable = false;
    enableElements = ['node', 'combo'];
    target = [];
    shadow;
    shadowOrigin = [0, 0];
    hiddenEdges = [];
    get animation() {
        if (!this.options.shadow)
            return false;
        return this.options.animation;
    }
    constructor(context, options) {
        super(context, Object.assign({}, DragElement.defaultOptions, options));
        this.onDragStart = this.onDragStart.bind(this);
        this.onDrag = this.onDrag.bind(this);
        this.onDragEnd = this.onDragEnd.bind(this);
        this.onDrop = this.onDrop.bind(this);
        this.bindEvents();
    }
    update(options) {
        super.update(options);
        this.bindEvents();
    }
    bindEvents() {
        const { graph } = this.context;
        this.unbindEvents();
        this.enableElements.forEach((type) => {
            graph.on(`${type}:${CommonEvent.DRAG_START}`, this.onDragStart);
            graph.on(`${type}:${CommonEvent.DRAG}`, this.onDrag);
            graph.on(`${type}:${CommonEvent.DRAG_END}`, this.onDragEnd);
        });
        if (['link'].includes(this.options.dropEffect)) {
            graph.on(`combo:${CommonEvent.DROP}`, this.onDrop);
            graph.on(`canvas:${CommonEvent.DROP}`, this.onDrop);
        }
    }
    getSelectedNodeIDs(currTarget) {
        return Array.from(new Set(this.context.graph
            .getElementDataByState('node', this.options.state)
            .map((node) => node.id)
            .concat(currTarget)));
    }
    getDelta(event) {
        const zoom = this.context.graph.getZoom();
        return divide([event.dx, event.dy], zoom);
    }
    onDragStart(event) {
        this.enable = this.validate(event);
        if (!this.enable)
            return;
        this.context.batch?.startBatch();
        this.target = this.getSelectedNodeIDs([event.target.id]);
        this.hideEdge();
        this.context.graph.frontElement(this.target);
        if (this.options.shadow)
            this.createShadow(this.target);
    }
    onDrag(event) {
        if (!this.enable)
            return;
        const delta = this.getDelta(event);
        if (this.options.shadow)
            this.moveShadow(delta);
        else
            this.moveElement(this.target, delta);
    }
    onDragEnd() {
        this.enable = false;
        if (this.options.shadow) {
            if (!this.shadow)
                return;
            this.shadow.style.visibility = 'hidden';
            const { x = 0, y = 0 } = this.shadow.attributes;
            const [dx, dy] = subtract([+x, +y], this.shadowOrigin);
            this.moveElement(this.target, [dx, dy]);
        }
        this.showEdges();
        this.options.onFinish?.(this.target);
        this.context.batch?.endBatch();
        this.target = [];
    }
    onDrop = async (event) => {
        if (this.options.dropEffect !== 'link')
            return;
        const { model, element } = this.context;
        const modifiedParentId = event.target.id;
        this.target.forEach((id) => {
            const originalParent = model.getParentData(id, COMBO_KEY);
            // 如果是在原父 combo 内部拖拽，需要刷新 combo 数据
            // If it is a drag and drop within the original parent combo, you need to refresh the combo data
            if (originalParent && idOf(originalParent) === modifiedParentId) {
                model.refreshComboData(modifiedParentId);
            }
            model.setParent(id, modifiedParentId, COMBO_KEY);
        });
        await element?.draw({ animation: true });
    };
    validate(event) {
        if (this.destroyed)
            return false;
        const { enable } = this.options;
        if (isFunction(enable))
            return enable(event);
        return !!enable;
    }
    async moveElement(ids, offset) {
        const { model, element } = this.context;
        const { dropEffect } = this.options;
        ids.forEach((id) => {
            const elementType = model.getElementType(id);
            if (elementType === 'node')
                model.translateNodeBy(id, offset);
            else if (elementType === 'combo')
                model.translateComboBy(id, offset);
        });
        if (dropEffect === 'move')
            ids.forEach((id) => model.refreshComboData(id));
        await element.draw({ animation: this.animation });
    }
    moveShadow(offset) {
        if (!this.shadow)
            return;
        const { x = 0, y = 0 } = this.shadow.attributes;
        const [dx, dy] = offset;
        this.shadow.attr({ x: +x + dx, y: +y + dy });
    }
    createShadow(target) {
        const shadowStyle = subStyleProps(this.options, 'shadow');
        const bbox = getCombinedBBox(target.map((id) => this.context.element.getElement(id).getBounds()));
        const [x, y] = bbox.min;
        this.shadowOrigin = [x, y];
        const [width, height] = getBBoxSize(bbox);
        const positionStyle = { width, height, x, y };
        if (this.shadow) {
            this.shadow.attr({
                ...shadowStyle,
                ...positionStyle,
                visibility: 'visible',
            });
        }
        else {
            this.shadow = new Rect({
                style: {
                    ...shadowStyle,
                    ...positionStyle,
                    pointerEvents: 'none',
                    // @ts-expect-error $layer is not in the type definition
                    $layer: 'transient',
                },
            });
            this.context.canvas.appendChild(this.shadow);
        }
    }
    showEdges() {
        if (this.options.shadow || this.hiddenEdges.length === 0)
            return;
        this.context.graph.showElement(this.hiddenEdges);
        this.hiddenEdges = [];
    }
    hideEdge() {
        const { hideEdge, shadow } = this.options;
        if (hideEdge === 'none' || shadow)
            return;
        const { graph } = this.context;
        if (hideEdge === 'all')
            this.hiddenEdges = graph.getEdgeData().map(idOf);
        else {
            this.hiddenEdges = Array.from(new Set(this.target.map((id) => graph.getRelatedEdgesData(id, hideEdge).map(idOf)).flat()));
        }
        graph.hideElement(this.hiddenEdges);
    }
    unbindEvents() {
        const { graph } = this.context;
        this.enableElements.forEach((type) => {
            graph.off(`${type}:${CommonEvent.DRAG_START}`, this.onDragStart);
            graph.off(`${type}:${CommonEvent.DRAG}`, this.onDrag);
            graph.off(`${type}:${CommonEvent.DRAG_END}`, this.onDragEnd);
        });
        graph.off(`combo:${CommonEvent.DROP}`, this.onDrop);
        graph.off(`canvas:${CommonEvent.DROP}`, this.onDrop);
    }
    destroy() {
        this.unbindEvents();
        this.shadow?.destroy();
        super.destroy();
    }
}
//# sourceMappingURL=drag-element.js.map