import { isFunction } from '@antv/util';
import { CommonEvent } from '../constants';
import { ELEMENT_TYPES } from '../constants/element';
import { idsOf } from '../utils/id';
import { getElementNthDegreeIds } from '../utils/relation';
import { BaseBehavior } from './base-behavior';
export class HoverElement extends BaseBehavior {
    static defaultOptions = {
        animation: true,
        enable: true,
        degree: 0,
        activeState: 'active',
        inactiveState: undefined,
    };
    constructor(context, options) {
        super(context, Object.assign({}, HoverElement.defaultOptions, options));
        this.bindEvents();
    }
    bindEvents() {
        const { graph } = this.context;
        this.unbindEvents();
        ELEMENT_TYPES.forEach((type) => {
            graph.on(`${type}:${CommonEvent.POINTER_OVER}`, this.hoverElement);
            graph.on(`${type}:${CommonEvent.POINTER_OUT}`, this.hoverEndElement);
        });
    }
    hoverElement = (event) => {
        if (!this.validate(event))
            return;
        this.updateElementsState(event, true);
        this.options.onHover?.(event);
    };
    hoverEndElement = (event) => {
        if (!this.validate(event))
            return;
        this.updateElementsState(event, false);
        this.options.onHoverEnd?.(event);
    };
    updateElementsState = (event, add) => {
        if (!this.options.activeState && !this.options.inactiveState)
            return;
        const { graph } = this.context;
        const { targetType, target } = event;
        const activeIds = getElementNthDegreeIds(graph, targetType, target.id, this.options.degree);
        const states = {};
        if (this.options.activeState) {
            Object.assign(states, this.getElementsState(activeIds, this.options.activeState, add));
        }
        if (this.options.inactiveState) {
            const inactiveIds = idsOf(graph.getData(), true).filter((id) => !activeIds.includes(id));
            Object.assign(states, this.getElementsState(inactiveIds, this.options.inactiveState, add));
        }
        graph.setElementState(states, this.options.animation);
    };
    getElementsState = (ids, state, add) => {
        const { graph } = this.context;
        const states = {};
        ids.forEach((id) => {
            const currentState = graph.getElementState(id);
            const updatedState = add ? [...currentState, state] : currentState.filter((s) => s !== state);
            states[id] = updatedState;
        });
        return states;
    };
    validate(event) {
        if (this.destroyed)
            return false;
        const { enable } = this.options;
        if (isFunction(enable))
            return enable(event);
        return !!enable;
    }
    unbindEvents() {
        const { graph } = this.context;
        ELEMENT_TYPES.forEach((type) => {
            graph.off(`${type}:${CommonEvent.POINTER_OVER}`, this.hoverElement);
            graph.off(`${type}:${CommonEvent.POINTER_OUT}`, this.hoverEndElement);
        });
    }
    destroy() {
        this.unbindEvents();
        super.destroy();
    }
}
//# sourceMappingURL=hover-element.js.map