import { Image, Path } from '@antv/g';
import { deepMix, isEmpty, isFunction } from '@antv/util';
import { getBBoxHeight, getBBoxWidth, getNodeBBox } from '../../utils/bbox';
import { getCubicLoopPath, getLabelPositionStyle } from '../../utils/edge';
import { findPorts, getConnectionPoint, isSameNode } from '../../utils/element';
import { omitStyleProps, subStyleProps } from '../../utils/prefix';
import { parseSize } from '../../utils/size';
import * as Symbol from '../../utils/symbol';
import { getWordWrapWidthByEnds } from '../../utils/text';
import { Label } from '../shapes';
import { BaseShape } from '../shapes/base-shape';
export class BaseEdge extends BaseShape {
    type = 'edge';
    static defaultStyleProps = {
        isBillboard: true,
        label: true,
        labelAutoRotate: true,
        labelIsBillboard: true,
        labelMaxWidth: '80%',
        labelOffsetX: 4,
        labelOffsetY: 0,
        labelPlacement: 'center',
        labelTextBaseline: 'middle',
        labelWordWrap: false,
        halo: false,
        haloDroppable: false,
        haloLineDash: 0,
        haloLineWidth: 12,
        haloPointerEvents: 'none',
        haloStrokeOpacity: 0.25,
        haloZIndex: -1,
        startArrow: false,
        startArrowAnchor: '0.5 0.5',
        startArrowSize: 8,
        startArrowLineDash: 0,
        startArrowLineWidth: 1,
        startArrowTransformOrigin: 'center',
        startArrowType: 'triangle',
        endArrow: false,
        endArrowAnchor: '0.5 0.5',
        endArrowSize: 8,
        endArrowLineDash: 0,
        endArrowLineWidth: 1,
        endArrowTransformOrigin: 'center',
        endArrowType: 'triangle',
        loopPlacement: 'top',
        loopClockwise: true,
    };
    constructor(options) {
        super(deepMix({}, { style: BaseEdge.defaultStyleProps }, options));
    }
    getKeyStyle(attributes) {
        const { sourceNode, targetNode, color, stroke, ...style } = this.getGraphicStyle(attributes);
        const path = isSameNode(sourceNode, targetNode) ? this.getLoopPath(attributes) : this.getKeyPath(attributes);
        return {
            path,
            ...omitStyleProps(style, ['halo', 'label', 'startArrow', 'endArrow']),
            stroke: color || stroke,
        };
    }
    getLoopPath(attributes) {
        const { sourceNode: node, sourcePort, targetPort } = attributes;
        const bbox = getNodeBBox(node);
        const defaultDist = Math.max(getBBoxWidth(bbox), getBBoxHeight(bbox));
        const { placement, clockwise, dist = defaultDist, } = subStyleProps(this.getGraphicStyle(attributes), 'loop');
        return getCubicLoopPath(node, placement, clockwise, dist, sourcePort, targetPort);
    }
    getEndpoints(attributes) {
        const { sourceNode, targetNode, sourcePort: sourcePortKey, targetPort: targetPortKey } = attributes;
        const [sourcePort, targetPort] = findPorts(sourceNode, targetNode, sourcePortKey, targetPortKey);
        const sourcePoint = getConnectionPoint(sourcePort || sourceNode, targetPort || targetNode);
        const targetPoint = getConnectionPoint(targetPort || targetNode, sourcePort || sourceNode);
        return [sourcePoint, targetPoint];
    }
    getHaloStyle(attributes) {
        if (attributes.halo === false)
            return false;
        const keyStyle = this.getKeyStyle(attributes);
        const haloStyle = subStyleProps(this.getGraphicStyle(attributes), 'halo');
        return { ...keyStyle, ...haloStyle };
    }
    getLabelStyle(attributes) {
        if (attributes.label === false || isEmpty(attributes.labelText))
            return false;
        const labelStyle = subStyleProps(this.getGraphicStyle(attributes), 'label');
        const { placement, offsetX, offsetY, autoRotate, maxWidth, ...restStyle } = labelStyle;
        const labelPositionStyle = getLabelPositionStyle(this.shapeMap.key, placement, autoRotate, offsetX, offsetY);
        const bbox = this.shapeMap.key.getLocalBounds();
        const wordWrapWidth = getWordWrapWidthByEnds([bbox.min, bbox.max], maxWidth);
        return Object.assign({ wordWrapWidth }, labelPositionStyle, restStyle);
    }
    drawArrow(attributes, type) {
        const isStart = type === 'start';
        const arrowType = type === 'start' ? 'startArrow' : 'endArrow';
        const enable = attributes[arrowType];
        const keyShape = this.shapeMap.key;
        if (enable) {
            const arrowStyle = this.getArrowStyle(attributes, isStart);
            const Ctor = !isEmpty(arrowStyle.src) ? Image : Path;
            const [marker, markerOffset, arrowOffset] = isStart
                ? ['markerStart', 'markerStartOffset', 'startArrowOffset']
                : ['markerEnd', 'markerEndOffset', 'endArrowOffset'];
            const arrow = keyShape.style[marker];
            // update
            if (arrow)
                arrow.attr(arrowStyle);
            // create
            else
                keyShape.style[marker] = new Ctor({ style: arrowStyle });
            keyShape.style[markerOffset] = attributes[arrowOffset] || arrowStyle.width / 2 + +arrowStyle.lineWidth;
        }
        else {
            // destroy
            const marker = isStart ? 'markerStart' : 'markerEnd';
            keyShape.style[marker]?.destroy();
            keyShape.style[marker] = null;
        }
    }
    getArrowStyle(attributes, isStart) {
        const keyStyle = this.getKeyStyle(attributes);
        const arrowType = isStart ? 'startArrow' : 'endArrow';
        const { size, type, ...arrowStyle } = subStyleProps(this.getGraphicStyle(attributes), arrowType);
        const [width, height] = parseSize(size);
        const arrowFn = isFunction(type) ? type : Symbol[type] || Symbol.triangle;
        const path = arrowFn(width, height);
        return {
            ...keyStyle,
            width,
            height,
            ...(path && { path, fill: type === 'simple' ? '' : keyStyle.stroke }),
            ...arrowStyle,
        };
    }
    drawLabelShape(attributes, container) {
        this.upsert('label', Label, this.getLabelStyle(attributes), container);
    }
    drawHaloShape(attributes, container) {
        this.upsert('halo', Path, this.getHaloStyle(attributes), container);
    }
    drawKeyShape(attributes, container) {
        return this.upsert('key', Path, this.getKeyStyle(attributes), container);
    }
    render(attributes = this.parsedAttributes, container = this) {
        // 1. key shape
        const keyShape = this.drawKeyShape(attributes, container);
        if (!keyShape)
            return;
        // 2. arrows
        this.drawArrow(attributes, 'start');
        this.drawArrow(attributes, 'end');
        // 3. label
        this.drawLabelShape(attributes, container);
        // 4. halo
        this.drawHaloShape(attributes, container);
    }
    /**
     * <zh/> 在元素完成创建并执行完入场动画后调用
     *
     * <en/> Called after the element is created and the entrance animation is completed
     * @override
     */
    onCreate() { }
    /**
     * <zh/> 在元素更新并执行完过渡动画后调用
     *
     * <en/> Called after the element is updated and the transition animation is completed
     * @override
     */
    onUpdate() { }
    /**
     * <zh/> 在元素完成退场动画并销毁后调用
     *
     * <en/> Called after the element completes the exit animation and is destroyed
     * @override
     */
    onDestroy() { }
    animate(keyframes, options) {
        const result = super.animate(keyframes, options);
        if (result) {
            result.onframe = () => {
                this.drawLabelShape(this.parsedAttributes, this);
            };
        }
        return result;
    }
}
//# sourceMappingURL=base-edge.js.map