import type { DisplayObject, DisplayObjectConfig, Group } from '@antv/g';
import type { ID } from '@antv/graphlib';
import type { CategoricalPalette } from '../../palettes/types';
import type { NodeData } from '../../spec';
import type { BadgePlacement, BaseElementStyleProps, Keyframe, LabelPlacement, Node, Placement, Point, Port, PortStyleProps, PrefixObject, Size } from '../../types';
import type { BadgeStyleProps, IconStyleProps, LabelStyleProps } from '../shapes';
import { BaseShape } from '../shapes';
export type BaseNodeStyleProps<T extends Record<string, unknown> = Record<string, any>> = T & BaseElementStyleProps & PrefixObject<NodeLabelStyleProps, 'label'> & PrefixObject<T, 'halo'> & PrefixObject<IconStyleProps, 'icon'> & PrefixObject<BadgeStyleProps, 'badge'> & PrefixObject<PortStyleProps, 'port'> & {
    /**
     * <zh/> x 坐标
     *
     * <en/> The x-coordinate of node
     */
    x?: number | string;
    /**
     * <zh/> y 坐标
     *
     * <en/> The y-coordinate of node
     */
    y?: number | string;
    /**
     * <zh/> z 坐标
     *
     * <en/> The z-coordinate of node
     */
    z?: number;
    /**
     * <zh/> 节点大小，快捷设置节点宽高
     *
     * <en/> The size of node, which is a shortcut to set the width and height of node
     */
    size?: Size;
    /**
     * <zh/> 主色
     *
     * <en/> Subject color
     */
    color?: string;
    /**
     * @deprecated
     * <zh/> 解决类型 style.getPropertyValue 问题，不要使用该属性
     *
     * <en/> Solve the problem of style.getPropertyValue, do not use this property
     * @description
     * <zh/> 移除该属性会导致 Polygon 及其子类与 Node 不兼容
     *
     * <en/> Removing this property will cause Polygon and its subclasses to be incompatible with Node
     * @ignore
     */
    points?: ([number, number] | [number, number, number])[];
    /**
     * <zh/> 父节点 id
     * <en/> The id of the parent node/combo
     */
    parentId?: ID;
    /**
     * <zh/> 是否收起
     *
     * <en/> Indicates whether the node is collapsed
     */
    collapsed?: boolean;
    /**
     * <zh/> 子节点实例
     *
     * <en/> The instance of the child node
     */
    childrenNode?: Node[];
    /**
     * <zh/> 子节点数据
     *
     * <en/> The data of the child node
     */
    childrenData?: NodeData[];
    /**
     * <zh/> 是否显示节点标签
     *
     * <en/> Whether to show the node label
     */
    label?: boolean;
    /**
     * <zh/> 是否显示节点光环
     *
     * <en/> Whether to show the node halo
     */
    halo?: boolean;
    /**
     * <zh/> 是否显示节点图标
     *
     * <en/> Whether to show the node icon
     */
    icon?: boolean;
    /**
     * <zh/> 是否显示节点徽标
     *
     * <en/> Whether to show the node badge
     */
    badge?: boolean;
    /**
     * <zh/> 是否显示连接桩
     *
     * <en/> Whether to show the node port
     */
    port?: boolean;
    /**
     * <zh/> 连接桩
     * <en/> Port
     */
    ports?: NodePortStyleProps[];
    /**
     * <zh/> 徽标
     * <en/> Badge
     */
    badges?: NodeBadgeStyleProps[];
    /**
     * <zh/> 徽标的背景色板
     * <en/> Badge background color palette
     */
    badgePalette?: string[] | CategoricalPalette;
};
/**
 * Design document: https://www.yuque.com/antv/g6/gl1iof1xpzg6ed98
 *
 * The P is the StyleProps of Key Shape.
 * The KeyShape is the type of the key shape.
 */
export declare abstract class BaseNode<S extends BaseNodeStyleProps = BaseNodeStyleProps> extends BaseShape<S> {
    type: string;
    static defaultStyleProps: Partial<BaseNodeStyleProps>;
    constructor(options: DisplayObjectConfig<S>);
    get parsedAttributes(): Required<S>;
    protected getSize(attributes?: S & import("@antv/g").BaseCustomElementStyleProps): import("../../types").Vector3;
    protected getKeyStyle(attributes: Required<S>): any;
    protected getLabelStyle(attributes: Required<S>): false | LabelStyleProps;
    protected getHaloStyle(attributes: Required<S>): any;
    protected getIconStyle(attributes: Required<S>): false | IconStyleProps;
    protected getBadgesStyle(attributes: Required<S>): Record<string, NodeBadgeStyleProps | false>;
    protected getBadgeStyle(style: NodeBadgeStyleProps): NodeBadgeStyleProps;
    protected getPortsStyle(attributes: Required<S>): Record<string, PortStyleProps | false>;
    protected getPortXY(attributes: Required<S>, style: NodePortStyleProps): Point;
    /**
     * Get the key shape for the node.
     * @returns Key shape.
     */
    getKey(): DisplayObject;
    /**
     * Get the ports for the node.
     * @returns Ports shape map.
     */
    getPorts(): Record<string, Port>;
    /**
     * Get the center point of the node.
     * @returns The center point of the node.
     */
    getCenter(): Point;
    /**
     * Get the point on the outer contour of the node that is the intersection with a line starting in the center the ending in the point `p`.
     * @param point - The point to intersect with the node.
     * @returns The intersection point.
     */
    getIntersectPoint(point: Point): Point;
    protected drawHaloShape(attributes: Required<S>, container: Group): void;
    protected drawBadgeShapes(attributes: Required<S>, container: Group): void;
    protected drawPortShapes(attributes: Required<S>, container: Group): void;
    protected abstract drawKeyShape(attributes: Required<S>, container: Group): DisplayObject | undefined;
    render(attributes?: Required<S>, container?: Group): void;
    /**
     * <zh/> 在元素完成创建并执行完入场动画后调用
     *
     * <en/> Called after the element is created and the entrance animation is completed
     * @override
     */
    onCreate(): void;
    /**
     * <zh/> 在元素更新并执行完过渡动画后调用
     *
     * <en/> Called after the element is updated and the transition animation is completed
     * @override
     */
    onUpdate(): void;
    /**
     * <zh/> 在元素完成退场动画并销毁后调用
     *
     * <en/> Called after the element completes the exit animation and is destroyed
     * @override
     */
    onDestroy(): void;
    animate(keyframes: Keyframe[], options?: number | KeyframeAnimationOptions): import("@antv/g").IAnimation | null;
}
type NodeLabelStyleProps = LabelStyleProps & {
    /**
     * Label position relative to the node (keyShape).
     */
    placement?: LabelPlacement;
    /**
     * The max width of the label, relative to the node width. The value can be a number or a percentage string:
     * If the value is a number, it will be treated as the pixel value.
     * If the value is a percentage string, it will be treated as the percentage of the node width.
     */
    maxWidth?: string | number;
};
type NodeBadgeStyleProps = BadgeStyleProps & {
    /**
     * Badge position relative to the node (keyShape).
     */
    placement?: BadgePlacement;
};
export type NodePortStyleProps = Partial<PortStyleProps> & {
    /**
     * The key of the port. Default is the index of the port.
     */
    key?: string;
    /**
     * The position of the port relative to the node (keyShape). The value can be a string or a tuple of two numbers.
     * If the value is a string, it will be treated as the position direction.
     * If the value is a tuple of two numbers, it will be treated as the position coordinates(0 ~ 1).
     */
    placement: Placement;
};
export {};
