import { Circle as GCircle } from '@antv/g';
import { deepMix, isEmpty } from '@antv/util';
import { getPortXYByPlacement, getTextStyleByPlacement, isSimplePort } from '../../utils/element';
import { getPaletteColors } from '../../utils/palette';
import { getRectIntersectPoint } from '../../utils/point';
import { getXYByPlacement } from '../../utils/position';
import { omitStyleProps, subObject, subStyleProps } from '../../utils/prefix';
import { parseSize } from '../../utils/size';
import { getWordWrapWidthByBox } from '../../utils/text';
import { Badge, BaseShape, Icon, Label } from '../shapes';
/**
 * Design document: https://www.yuque.com/antv/g6/gl1iof1xpzg6ed98
 *
 * The P is the StyleProps of Key Shape.
 * The KeyShape is the type of the key shape.
 */
export class BaseNode extends BaseShape {
    type = 'node';
    static defaultStyleProps = {
        x: 0,
        y: 0,
        size: 24,
        droppable: true,
        draggable: true,
        port: true,
        ports: [],
        portZIndex: 2,
        portLinkToCenter: false,
        badge: true,
        badges: [],
        badgeZIndex: 3,
        halo: false,
        haloDroppable: false,
        haloLineDash: 0,
        haloLineWidth: 12,
        haloStrokeOpacity: 0.25,
        haloPointerEvents: 'none',
        haloZIndex: -1,
        icon: true,
        iconZIndex: 1,
        label: true,
        labelIsBillboard: true,
        labelMaxWidth: '200%',
        labelPlacement: 'bottom',
        labelWordWrap: false,
        labelZIndex: 0,
    };
    constructor(options) {
        super(deepMix({}, { style: BaseNode.defaultStyleProps }, options));
    }
    get parsedAttributes() {
        return this.attributes;
    }
    getSize(attributes = this.attributes) {
        const { size } = attributes;
        return parseSize(size);
    }
    getKeyStyle(attributes) {
        const { color, fill, ...style } = this.getGraphicStyle(attributes);
        return Object.assign({ fill: color || fill }, omitStyleProps(style, ['label', 'halo', 'icon', 'badge', 'port']));
    }
    getLabelStyle(attributes) {
        if (attributes.label === false || isEmpty(attributes.labelText))
            return false;
        const { placement, maxWidth, ...labelStyle } = subStyleProps(this.getGraphicStyle(attributes), 'label');
        const keyShape = this.getKey();
        const keyBounds = keyShape.getLocalBounds();
        return Object.assign(getTextStyleByPlacement(keyBounds, placement), { wordWrapWidth: getWordWrapWidthByBox(keyBounds, maxWidth) }, labelStyle);
    }
    getHaloStyle(attributes) {
        if (attributes.halo === false)
            return false;
        const { fill, ...keyStyle } = this.getKeyStyle(attributes);
        const haloStyle = subStyleProps(this.getGraphicStyle(attributes), 'halo');
        return { ...keyStyle, stroke: fill, ...haloStyle };
    }
    getIconStyle(attributes) {
        if (attributes.icon === false || isEmpty(attributes.iconText || attributes.iconSrc))
            return false;
        const iconStyle = subStyleProps(this.getGraphicStyle(attributes), 'icon');
        const keyShape = this.getKey();
        const [x, y] = getXYByPlacement(keyShape.getLocalBounds(), 'center');
        return { x, y, ...iconStyle };
    }
    getBadgesStyle(attributes) {
        const badges = subObject(this.shapeMap, 'badge-');
        const badgesShapeStyle = {};
        Object.keys(badges).forEach((key) => {
            badgesShapeStyle[key] = false;
        });
        if (attributes.badge === false || isEmpty(attributes.badges))
            return badgesShapeStyle;
        const { badges: badgeOptions = [], badgePalette, ...restAttributes } = attributes;
        const colors = getPaletteColors(badgePalette);
        const badgeStyle = subStyleProps(this.getGraphicStyle(restAttributes), 'badge');
        badgeOptions.forEach((option, i) => {
            badgesShapeStyle[i] = {
                backgroundFill: colors ? colors[i % colors?.length] : undefined,
                ...badgeStyle,
                ...this.getBadgeStyle(option),
            };
        });
        return badgesShapeStyle;
    }
    getBadgeStyle(style) {
        const keyShape = this.getKey();
        const { placement = 'top', ...restStyle } = style;
        const textStyle = getTextStyleByPlacement(keyShape.getLocalBounds(), placement, true);
        return { ...textStyle, ...restStyle };
    }
    getPortsStyle(attributes) {
        const ports = this.getPorts();
        const portsShapeStyle = {};
        Object.keys(ports).forEach((key) => {
            portsShapeStyle[key] = false;
        });
        if (attributes.port === false || isEmpty(attributes.ports))
            return portsShapeStyle;
        const portStyle = subStyleProps(this.getGraphicStyle(attributes), 'port');
        const { ports: portOptions = [] } = attributes;
        portOptions.forEach((option, index) => {
            const key = option.key || index;
            const mergedStyle = { ...portStyle, ...option };
            if (isSimplePort(mergedStyle)) {
                portsShapeStyle[key] = false;
            }
            else {
                const [x, y] = this.getPortXY(attributes, option);
                portsShapeStyle[key] = { cx: x, cy: y, ...mergedStyle };
            }
        });
        return portsShapeStyle;
    }
    getPortXY(attributes, style) {
        const { placement = 'left' } = style;
        const bounds = this.getKey().getLocalBounds();
        return getPortXYByPlacement(bounds, placement);
    }
    /**
     * Get the key shape for the node.
     * @returns Key shape.
     */
    getKey() {
        return this.shapeMap.key;
    }
    /**
     * Get the ports for the node.
     * @returns Ports shape map.
     */
    getPorts() {
        return subObject(this.shapeMap, 'port-');
    }
    /**
     * Get the center point of the node.
     * @returns The center point of the node.
     */
    getCenter() {
        return this.getKey().getBounds().center;
    }
    /**
     * Get the point on the outer contour of the node that is the intersection with a line starting in the center the ending in the point `p`.
     * @param point - The point to intersect with the node.
     * @returns The intersection point.
     */
    getIntersectPoint(point) {
        const keyShapeBounds = this.getKey().getBounds();
        return getRectIntersectPoint(point, keyShapeBounds);
    }
    drawHaloShape(attributes, container) {
        const keyShape = this.getKey();
        this.upsert('halo', keyShape.constructor, this.getHaloStyle(attributes), container);
    }
    drawBadgeShapes(attributes, container) {
        const badgesStyle = this.getBadgesStyle(attributes);
        Object.keys(badgesStyle).forEach((key) => {
            this.upsert(`badge-${key}`, Badge, badgesStyle[key], container);
        });
    }
    drawPortShapes(attributes, container) {
        const portsStyle = this.getPortsStyle(attributes);
        Object.keys(portsStyle).forEach((key) => {
            this.upsert(`port-${key}`, GCircle, portsStyle[key], container);
        });
    }
    render(attributes = this.parsedAttributes, container = this) {
        // 1. key shape
        const keyShape = this.drawKeyShape(attributes, container);
        if (!keyShape)
            return;
        // 2. halo, use shape same with keyShape
        this.drawHaloShape(attributes, container);
        // 3. icon
        this.upsert('icon', Icon, this.getIconStyle(attributes), container);
        // 4. badges
        this.drawBadgeShapes(attributes, container);
        // 5. label
        this.upsert('label', Label, this.getLabelStyle(attributes), container);
        // 6. ports
        this.drawPortShapes(attributes, container);
    }
    /**
     * <zh/> 在元素完成创建并执行完入场动画后调用
     *
     * <en/> Called after the element is created and the entrance animation is completed
     * @override
     */
    onCreate() { }
    /**
     * <zh/> 在元素更新并执行完过渡动画后调用
     *
     * <en/> Called after the element is updated and the transition animation is completed
     * @override
     */
    onUpdate() { }
    /**
     * <zh/> 在元素完成退场动画并销毁后调用
     *
     * <en/> Called after the element completes the exit animation and is destroyed
     * @override
     */
    onDestroy() { }
    animate(keyframes, options) {
        const result = super.animate(keyframes, options);
        if (result) {
            result.onframe = () => {
                this.drawBadgeShapes(this.parsedAttributes, this);
            };
        }
        return result;
    }
}
//# sourceMappingURL=base-node.js.map