import { Tooltip as TooltipComponent } from '@antv/component';
import { get } from '@antv/util';
import { BasePlugin } from './base-plugin';
export class Tooltip extends BasePlugin {
    static defaultOptions = {
        trigger: 'hover',
        position: 'top-right',
        enterable: false,
        enable: true,
        style: {
            '.tooltip': {
                visibility: 'hidden',
            },
        },
    };
    currentTarget = null;
    tooltipElement = null;
    container = null;
    constructor(context, options) {
        super(context, Object.assign({}, Tooltip.defaultOptions, options));
        this.render();
        this.bindEvents();
    }
    getEvents() {
        if (this.options.trigger === 'click') {
            return {
                'node:click': this.onClick,
                'edge:click': this.onClick,
                'combo:click': this.onClick,
                'canvas:click': this.onPointerLeave,
                contextmenu: this.onPointerLeave,
                drag: this.onPointerLeave,
            };
        }
        return {
            'node:pointerenter': this.onPointerEnter,
            'node:pointermove': this.onPointerMove,
            'canvas:pointermove': this.onCanvasMove,
            'edge:pointerenter': this.onPointerEnter,
            'edge:pointermove': this.onPointerMove,
            'combo:pointerenter': this.onPointerEnter,
            'combo:pointermove': this.onPointerMove,
            contextmenu: this.onPointerLeave,
            'node:drag': this.onPointerLeave,
        };
    }
    update(options) {
        this.unbindEvents();
        super.update(options);
        if (this.tooltipElement) {
            this.container?.removeChild(this.tooltipElement.HTMLTooltipElement);
        }
        this.tooltipElement = this.initTooltip();
        this.bindEvents();
    }
    render() {
        const { canvas } = this.context;
        const $container = canvas.getContainer();
        if (!$container)
            return;
        this.container = $container;
        this.tooltipElement = this.initTooltip();
    }
    unbindEvents() {
        const { graph } = this.context;
        /** The previous event binding needs to be removed when updating the trigger. */
        const events = this.getEvents();
        Object.keys(events).forEach((eventName) => {
            graph.off(eventName, events[eventName]);
        });
    }
    bindEvents() {
        const { graph } = this.context;
        const events = this.getEvents();
        Object.keys(events).forEach((eventName) => {
            graph.on(eventName, events[eventName]);
        });
    }
    isEnable = (e) => {
        const { enable } = this.options;
        if (typeof enable === 'function') {
            return enable(e);
        }
        return enable;
    };
    onClick = (e) => {
        const { target: { id }, } = e;
        // click the same item twice, tooltip will be hidden
        if (this.currentTarget === id) {
            this.currentTarget = null;
            this.hideTooltip(e);
        }
        else {
            this.currentTarget = id;
            this.showTooltip(e);
        }
    };
    onPointerMove = (e) => {
        const { target } = e;
        if (!this.currentTarget || target.id === this.currentTarget) {
            return;
        }
        this.showTooltip(e);
    };
    onPointerLeave = (e) => {
        this.hideTooltip(e);
        this.currentTarget = null;
    };
    onCanvasMove = (e) => {
        this.hideTooltip(e);
        this.currentTarget = null;
    };
    onPointerEnter = (e) => {
        this.showTooltip(e);
    };
    showTooltipById = (id, elementType = 'node') => {
        const e = {
            targetType: elementType,
            target: { id },
        };
        this.showTooltip(e);
    };
    getItems = (id, targetType) => {
        const { model } = this.context;
        switch (targetType) {
            case 'node':
                return model.getNodeData([id]);
            case 'edge':
                return model.getEdgeData([id]);
            case 'combo':
                return model.getComboData([id]);
            default:
                return [];
        }
    };
    showTooltip = (e) => {
        const { targetType, client, target: { id, attributes = { color: '#1883FF' } }, } = e;
        if (!this.tooltipElement || !this.isEnable(e))
            return;
        const { getContent } = this.options;
        const { color, stroke } = attributes;
        this.currentTarget = id;
        const items = this.getItems(id, targetType);
        let x;
        let y;
        if (client) {
            x = client.x;
            y = client.y;
        }
        else {
            const style = get(items, '0.style', { x: 0, y: 0 });
            x = style.x;
            y = style.y;
        }
        let tooltipContent = {};
        if (getContent) {
            tooltipContent.content = getContent(e, items);
        }
        else {
            tooltipContent = {
                title: targetType,
                data: items.map((item) => {
                    return {
                        name: 'ID',
                        value: item.id || `${item.source} -> ${item.target}`,
                        color: color || stroke,
                    };
                }),
            };
        }
        this.tooltipElement.update({
            x,
            y,
            style: {
                '.tooltip': {
                    visibility: 'visible',
                },
            },
            ...tooltipContent,
        });
    };
    hideTooltip = (e) => {
        // if e is undefined, hide the tooltip， external call
        if (!e) {
            this.tooltipElement?.hide();
            return;
        }
        if (!this.tooltipElement || !this.isEnable(e))
            return;
        const { client: { x, y }, } = e;
        this.tooltipElement.hide(x, y);
    };
    initTooltip = () => {
        const { canvas } = this.context;
        const { center } = canvas.getBounds();
        const $container = canvas.getContainer();
        const { top, left } = $container.getBoundingClientRect();
        const { style, position, enterable, container = { x: -left, y: -top } } = this.options;
        const [x, y] = center;
        const [width, height] = canvas.getSize();
        const tooltipElement = new TooltipComponent({
            className: 'tooltip',
            style: {
                x,
                y,
                container,
                bounding: {
                    x: 0,
                    y: 0,
                    width,
                    height,
                },
                position,
                enterable,
                title: '',
                offset: [10, 10],
                style,
            },
        });
        this.container?.appendChild(tooltipElement.HTMLTooltipElement);
        return tooltipElement;
    };
    destroy() {
        this.unbindEvents();
        if (this.tooltipElement) {
            this.container?.removeChild(this.tooltipElement.HTMLTooltipElement);
        }
        super.destroy();
    }
}
//# sourceMappingURL=tooltip.js.map