import { Graph as GraphLib } from '@antv/graphlib';
import { isEqual, isUndefined } from '@antv/util';
import { COMBO_KEY, ChangeType, TREE_KEY } from '../constants';
import { cloneElementData, mergeElementsData } from '../utils/data';
import { arrayDiff } from '../utils/diff';
import { toG6Data, toGraphlibData } from '../utils/graphlib';
import { idOf, parentIdOf } from '../utils/id';
import { positionOf } from '../utils/position';
import { zIndexOf } from '../utils/style';
import { dfs } from '../utils/traverse';
import { add } from '../utils/vector';
export class DataController {
    model;
    /**
     * <zh/> 最近一次删除的 combo 的 id
     *
     * <en/> The ids of the last deleted combos
     * @description
     * <zh/> 当删除 combo 后，会将其 id 从 comboIds 中移除，此时根据 Graphlib 的 changes 事件获取到的 NodeRemoved 无法区分是 combo 还是 node。
     * 因此需要记录最近一次删除的 combo 的 id，并用于 isCombo 的判断
     *
     * <en/> When the combo is deleted, its id will be removed from comboIds. At this time, the NodeRemoved obtained according to the changes event of Graphlib cannot distinguish whether it is a combo or a node.
     * Therefore, it is necessary to record the id of the last deleted combo and use it to judge isCombo
     */
    latestRemovedComboIds = new Set();
    comboIds = new Set();
    /**
     * <zh/> 获取详细数据变更
     *
     * <en/> Get detailed data changes
     */
    changes = [];
    /**
     * <zh/> 批处理计数器
     *
     * <en/> Batch processing counter
     */
    batchCount = 0;
    /**
     * <zh/> 是否处于无痕模式
     *
     * <en/> Whether it is in traceless mode
     */
    isTraceless = false;
    constructor() {
        this.model = new GraphLib();
    }
    pushChange(change) {
        if (this.isTraceless)
            return;
        const { type } = change;
        if (type === ChangeType.NodeUpdated || type === ChangeType.EdgeUpdated || type === ChangeType.ComboUpdated) {
            const { value, original } = change;
            this.changes.push({ value: cloneElementData(value), original: cloneElementData(original), type });
        }
        else {
            this.changes.push({ value: cloneElementData(change.value), type });
        }
    }
    /**
     * <zh/> [警告] 此 API 仅供 Element Controller 调用
     *
     * <en/> [WARNING] This API is only for Element Controller
     * @returns <zh/> 数据变更 | <en/> data changes
     */
    getChanges() {
        return this.changes;
    }
    clearChanges() {
        this.changes = [];
    }
    batch(callback) {
        this.batchCount++;
        this.model.batch(callback);
        this.batchCount--;
    }
    /**
     * <zh/> 执行操作而不会留下记录
     *
     * <en/> Perform operations without leaving records
     * @param callback - <zh/> 回调函数 | <en/> callback function
     * @description
     * <zh/> 通常用于运行时调整元素并同步数据，避免触发数据变更导致重绘
     *
     * <en/> Usually used to adjust elements at runtime and synchronize data to avoid triggering data changes and causing redraws
     */
    silence(callback) {
        this.isTraceless = true;
        callback();
        this.isTraceless = false;
    }
    isCombo(id) {
        return this.comboIds.has(id) || this.latestRemovedComboIds.has(id);
    }
    getData() {
        return {
            nodes: this.getNodeData(),
            edges: this.getEdgeData(),
            combos: this.getComboData(),
        };
    }
    getNodeData(ids) {
        return this.model.getAllNodes().reduce((acc, node) => {
            const data = toG6Data(node);
            if (this.isCombo(idOf(data)))
                return acc;
            if (ids === undefined)
                acc.push(data);
            else
                ids.includes(idOf(data)) && acc.push(data);
            return acc;
        }, []);
    }
    getEdgeData(ids) {
        return this.model.getAllEdges().reduce((acc, edge) => {
            const data = toG6Data(edge);
            if (ids === undefined)
                acc.push(data);
            else
                ids.includes(idOf(data)) && acc.push(data);
            return acc;
        }, []);
    }
    getComboData(ids) {
        return this.model.getAllNodes().reduce((acc, combo) => {
            const data = toG6Data(combo);
            if (!this.isCombo(idOf(data)))
                return acc;
            if (ids === undefined)
                acc.push(data);
            else
                ids.includes(idOf(data)) && acc.push(data);
            return acc;
        }, []);
    }
    getAncestorsData(id, hierarchy) {
        const { model } = this;
        if (!model.hasNode(id) || !model.hasTreeStructure(hierarchy))
            return [];
        return model.getAncestors(id, hierarchy).map(toG6Data);
    }
    getDescendantsData(id) {
        const root = this.getElementsData([id])[0];
        const data = [];
        dfs(root, (node) => {
            if (node !== root)
                data.push(node);
        }, (node) => this.getChildrenData(idOf(node)), 'TB');
        return data;
    }
    getParentData(id, hierarchy) {
        const { model } = this;
        if (!hierarchy) {
            console.error('The hierarchy structure key is not specified');
            return undefined;
        }
        if (!model.hasNode(id) || !model.hasTreeStructure(hierarchy))
            return undefined;
        const parent = model.getParent(id, hierarchy);
        return parent ? toG6Data(parent) : undefined;
    }
    getChildrenData(id) {
        const structureKey = this.getElementType(id) === 'node' ? TREE_KEY : COMBO_KEY;
        const { model } = this;
        if (!model.hasNode(id) || !model.hasTreeStructure(structureKey))
            return [];
        return model.getChildren(id, structureKey).map(toG6Data);
    }
    /**
     * <zh/> 获取指定类型元素的数据
     *
     * <en/> Get the data of the specified type of element
     * @param elementType - <zh/> 元素类型 | <en/> element type
     * @returns <zh/> 元素数据 | <en/> element data
     */
    getElementData(elementType) {
        if (elementType === 'node')
            return this.getNodeData();
        if (elementType === 'edge')
            return this.getEdgeData();
        if (elementType === 'combo')
            return this.getComboData();
        return [];
    }
    /**
     * <zh/> 根据 ID 获取元素的数据，不用关心元素的类型
     *
     * <en/> Get the data of the element by ID, no need to care about the type of the element
     * @param ids - <zh/> 元素 ID 数组 | <en/> element ID array
     * @returns <zh/> 元素数据 | <en/> data of the element
     */
    getElementsData(ids) {
        return ids.map((id) => {
            const type = this.getElementType(id);
            if (type === 'node')
                return this.getNodeData([id])[0];
            else if (type === 'edge')
                return this.getEdgeData([id])[0];
            return this.getComboData([id])[0];
        });
    }
    /**
     * <zh/> 获取所有节点和 combo 的数据
     *
     * <en/> Get all node and combo data
     * @param ids - <zh/> 节点和 combo ID 数组 | <en/> node and combo ID array
     * @returns <zh/> 节点和 combo 的数据 | <en/> node and combo data
     */
    getNodeLikeData(ids) {
        return this.model.getAllNodes().reduce((acc, node) => {
            const data = toG6Data(node);
            if (ids)
                ids.includes(idOf(data)) && acc.push(data);
            else
                acc.push(data);
            return acc;
        }, []);
    }
    getElementDataByState(elementType, state) {
        const elementData = this.getElementData(elementType);
        return elementData.filter((datum) => datum.style?.states?.includes(state));
    }
    getElementState(id) {
        return this.getElementsData([id])?.[0]?.style?.states || [];
    }
    hasNode(id) {
        return this.model.hasNode(id) && !this.isCombo(id);
    }
    hasEdge(id) {
        return this.model.hasEdge(id);
    }
    hasCombo(id) {
        return this.model.hasNode(id) && this.isCombo(id);
    }
    getRelatedEdgesData(id, direction = 'both') {
        return this.model.getRelatedEdges(id, direction).map(toG6Data);
    }
    getNeighborNodesData(id) {
        return this.model.getNeighbors(id).map(toG6Data);
    }
    setData(data) {
        const { nodes: modifiedNodes = [], edges: modifiedEdges = [], combos: modifiedCombos = [] } = data;
        const { nodes: originalNodes = [], edges: originalEdges = [], combos: originalCombos = [] } = this.getData();
        const nodeDiff = arrayDiff(originalNodes, modifiedNodes, (node) => idOf(node));
        const edgeDiff = arrayDiff(originalEdges, modifiedEdges, (edge) => idOf(edge));
        const comboDiff = arrayDiff(originalCombos, modifiedCombos, (combo) => idOf(combo));
        this.batch(() => {
            this.addData({
                nodes: nodeDiff.enter,
                edges: edgeDiff.enter,
                combos: comboDiff.enter,
            });
            this.updateData({
                nodes: nodeDiff.update,
                edges: edgeDiff.update,
                combos: comboDiff.update,
            });
            this.removeData({
                nodes: nodeDiff.exit.map(idOf),
                edges: edgeDiff.exit.map(idOf),
                combos: comboDiff.exit.map(idOf),
            });
        });
    }
    addData(data) {
        const { nodes, edges, combos } = data;
        this.batch(() => {
            // add combo first
            this.addComboData(combos);
            this.addNodeData(nodes);
            this.addEdgeData(edges);
        });
    }
    addNodeData(nodes = []) {
        if (!nodes.length)
            return;
        this.model.addNodes(nodes.map((node) => {
            this.pushChange({ value: node, type: ChangeType.NodeAdded });
            return toGraphlibData(node);
        }));
        this.updateNodeLikeHierarchy(nodes);
    }
    addEdgeData(edges = []) {
        if (!edges.length)
            return;
        this.model.addEdges(edges.map((edge) => {
            this.pushChange({ value: edge, type: ChangeType.EdgeAdded });
            return toGraphlibData(edge);
        }));
    }
    addComboData(combos = []) {
        if (!combos.length)
            return;
        const { model } = this;
        if (!model.hasTreeStructure(COMBO_KEY)) {
            model.attachTreeStructure(COMBO_KEY);
        }
        model.addNodes(combos.map((combo) => {
            this.comboIds.add(idOf(combo));
            this.pushChange({ value: combo, type: ChangeType.ComboAdded });
            return toGraphlibData(combo);
        }));
        this.updateNodeLikeHierarchy(combos);
    }
    updateNodeLikeHierarchy(data) {
        if (!this.enableUpdateNodeLikeHierarchy)
            return;
        const { model } = this;
        data.forEach((datum) => {
            const id = idOf(datum);
            model.attachTreeStructure(COMBO_KEY);
            this.setParent(id, parentIdOf(datum), COMBO_KEY);
            const children = datum?.style?.children;
            if (children !== undefined) {
                model.attachTreeStructure(TREE_KEY);
                children.forEach((child) => {
                    this.setParent(child, id, TREE_KEY);
                });
            }
        });
    }
    enableUpdateNodeLikeHierarchy = true;
    /**
     * <zh/> 执行变更时不要更新节点层次结构
     *
     * <en/> Do not update the node hierarchy when executing changes
     * @param callback - <zh/> 变更函数 | <en/> change function
     */
    preventUpdateNodeLikeHierarchy(callback) {
        this.enableUpdateNodeLikeHierarchy = false;
        callback();
        this.enableUpdateNodeLikeHierarchy = true;
    }
    updateData(data) {
        const { nodes, edges, combos } = data;
        this.batch(() => {
            this.updateNodeData(nodes);
            this.updateComboData(combos);
            this.updateEdgeData(edges);
        });
    }
    updateNodeData(nodes = []) {
        if (!nodes.length)
            return;
        const { model } = this;
        this.batch(() => {
            const modifiedNodes = [];
            nodes.forEach((modifiedNode) => {
                const id = idOf(modifiedNode);
                const originalNode = toG6Data(model.getNode(id));
                if (isEqual(originalNode, modifiedNode))
                    return;
                const value = mergeElementsData(originalNode, modifiedNode);
                this.pushChange({ value, original: originalNode, type: ChangeType.NodeUpdated });
                model.mergeNodeData(id, value);
                modifiedNodes.push(value);
            });
            this.updateNodeLikeHierarchy(modifiedNodes);
        });
    }
    syncNodeDatum(datum) {
        const { model } = this;
        const id = idOf(datum);
        const original = toG6Data(model.getNode(id));
        const value = mergeElementsData(original, datum);
        model.mergeNodeData(id, value);
    }
    updateEdgeData(edges = []) {
        if (!edges.length)
            return;
        const { model } = this;
        this.batch(() => {
            edges.forEach((modifiedEdge) => {
                const id = idOf(modifiedEdge);
                const originalEdge = toG6Data(model.getEdge(id));
                if (isEqual(originalEdge, modifiedEdge))
                    return;
                if (modifiedEdge.source && originalEdge.source !== modifiedEdge.source) {
                    model.updateEdgeSource(id, modifiedEdge.source);
                }
                if (modifiedEdge.target && originalEdge.target !== modifiedEdge.target) {
                    model.updateEdgeTarget(id, modifiedEdge.target);
                }
                const updatedData = mergeElementsData(originalEdge, modifiedEdge);
                this.pushChange({ value: updatedData, original: originalEdge, type: ChangeType.EdgeUpdated });
                model.mergeEdgeData(id, updatedData);
            });
        });
    }
    updateComboData(combos = []) {
        if (!combos.length)
            return;
        const { model } = this;
        model.batch(() => {
            const modifiedCombos = [];
            combos.forEach((modifiedCombo) => {
                const id = idOf(modifiedCombo);
                const originalCombo = toG6Data(model.getNode(id));
                if (isEqual(originalCombo, modifiedCombo))
                    return;
                const value = mergeElementsData(originalCombo, modifiedCombo);
                this.pushChange({ value, original: originalCombo, type: ChangeType.ComboUpdated });
                model.mergeNodeData(id, value);
                modifiedCombos.push(value);
            });
            this.updateNodeLikeHierarchy(modifiedCombos);
        });
    }
    /**
     * <zh/> 设置节点的父节点
     *
     * <en/> Set the parent node of the node
     * @param id - <zh/> 节点 ID | <en/> node ID
     * @param parentId - <zh/> 父节点 ID | <en/> parent node ID
     * @param hierarchy - <zh/> 层次结构类型 | <en/> hierarchy type
     * @param update - <zh/> 添加新/旧父节点数据更新记录 | <en/> add new/old parent node data update record
     */
    setParent(id, parentId, hierarchy, update = true) {
        if (id === parentId)
            return;
        const originalParentId = parentIdOf(this.getNodeLikeData([id])[0]);
        // Sync data
        if (originalParentId !== parentId) {
            const modifiedDatum = { id, style: { parentId } };
            if (this.isCombo(id))
                this.syncComboDatum(modifiedDatum);
            else
                this.syncNodeDatum(modifiedDatum);
        }
        this.model.setParent(id, parentId, hierarchy);
        if (update) {
            new Set([originalParentId, parentId]).forEach((pId) => {
                if (pId !== undefined)
                    this.refreshComboData(pId);
            });
        }
    }
    /**
     * <zh/> 刷新 combo 数据
     *
     * <en/> Refresh combo data
     * @param id - <zh/> combo ID | <en/> combo ID
     * @description
     * <zh/> 不会更改数据，但会触发数据变更事件
     *
     * <en/> Will not change the data, but will trigger data change events
     */
    refreshComboData(id) {
        const combo = this.getComboData([id])[0];
        const ancestors = this.getAncestorsData(id, COMBO_KEY);
        if (combo)
            this.pushChange({ value: combo, original: combo, type: ChangeType.ComboUpdated });
        ancestors.forEach((value) => {
            this.pushChange({ value: value, original: value, type: ChangeType.ComboUpdated });
        });
    }
    /**
     * <zh/> 将 combo 数据同步到 model 中
     *
     * <en/> Synchronize combo data to the model
     * @param datum - <zh/> combo 数据 | <en/> combo data
     */
    syncComboDatum(datum) {
        const { model } = this;
        const id = idOf(datum);
        const original = toG6Data(model.getNode(id));
        const value = mergeElementsData(original, datum);
        model.mergeNodeData(id, value);
    }
    getElementPosition(id) {
        const datum = this.getElementsData([id])[0];
        return positionOf(datum);
    }
    translateNodeBy(id, offset) {
        const curr = this.getElementPosition(id);
        const position = add(curr, [...offset, 0].slice(0, 3));
        this.translateNodeTo(id, position);
    }
    translateNodeTo(id, position) {
        const [x = 0, y = 0, z = 0] = position;
        this.preventUpdateNodeLikeHierarchy(() => {
            this.updateNodeData([{ id, style: { x, y, z } }]);
        });
    }
    translateComboBy(id, offset) {
        const [dx = 0, dy = 0, dz = 0] = offset;
        if ([dx, dy, dz].some(isNaN) || [dx, dy, dz].every((o) => o === 0))
            return;
        const combo = this.getComboData([id])[0];
        if (!combo)
            return;
        dfs(combo, (succeed) => {
            const succeedID = idOf(succeed);
            const [x, y, z] = positionOf(succeed);
            const value = mergeElementsData(succeed, {
                style: { x: x + dx, y: y + dy, z: z + dz },
            });
            this.pushChange({
                value,
                // @ts-ignore
                original: succeed,
                type: this.isCombo(succeedID) ? ChangeType.ComboUpdated : ChangeType.NodeUpdated,
            });
            this.model.mergeNodeData(succeedID, value);
        }, (node) => this.getChildrenData(idOf(node)), 'BT');
    }
    translateComboTo(id, position) {
        if (position.some(isNaN))
            return;
        const [tx = 0, ty = 0, tz = 0] = position;
        const combo = this.getComboData([id])?.[0];
        if (!combo)
            return;
        const [comboX, comboY, comboZ] = positionOf(combo);
        const dx = tx - comboX;
        const dy = ty - comboY;
        const dz = tz - comboZ;
        dfs(combo, (succeed) => {
            const succeedId = idOf(succeed);
            const [x, y, z] = positionOf(succeed);
            const value = mergeElementsData(succeed, {
                style: { x: x + dx, y: y + dy, z: z + dz },
            });
            this.pushChange({
                value,
                // @ts-ignore
                original: succeed,
                type: this.isCombo(succeedId) ? ChangeType.ComboUpdated : ChangeType.NodeUpdated,
            });
            this.model.mergeNodeData(succeedId, value);
        }, (node) => this.getChildrenData(idOf(node)), 'BT');
    }
    removeData(data) {
        const { nodes, edges, combos } = data;
        this.batch(() => {
            // remove edges first
            this.removeEdgeData(edges);
            this.removeNodeData(nodes);
            this.removeComboData(combos);
            this.latestRemovedComboIds = new Set(combos);
        });
    }
    removeNodeData(ids = []) {
        if (!ids.length)
            return;
        this.batch(() => {
            ids.forEach((id) => {
                // 移除关联边、子节点
                // remove related edges and child nodes
                this.removeEdgeData(this.getRelatedEdgesData(id).map(idOf));
                // TODO 树图情况下移除子节点
                this.pushChange({ value: this.getNodeData([id])[0], type: ChangeType.NodeRemoved });
                this.removeNodeLikeHierarchy(id);
            });
            this.model.removeNodes(ids);
        });
    }
    removeEdgeData(ids = []) {
        if (!ids.length)
            return;
        ids.forEach((id) => this.pushChange({ value: this.getEdgeData([id])[0], type: ChangeType.EdgeRemoved }));
        this.model.removeEdges(ids);
    }
    removeComboData(ids = []) {
        if (!ids.length)
            return;
        this.batch(() => {
            ids.forEach((id) => {
                this.pushChange({ value: this.getComboData([id])[0], type: ChangeType.ComboRemoved });
                this.removeNodeLikeHierarchy(id);
                this.comboIds.delete(id);
            });
            this.model.removeNodes(ids);
        });
    }
    /**
     * <zh/> 移除节点层次结构，将其子节点移动到父节点的 children 列表中
     *
     * <en/> Remove the node hierarchy and move its child nodes to the parent node's children list
     * @param id - <zh/> 待处理的节点 | <en/> node to be processed
     */
    removeNodeLikeHierarchy(id) {
        if (this.model.hasTreeStructure(COMBO_KEY)) {
            const grandParentId = parentIdOf(this.getNodeLikeData([id])[0]);
            // 从父节点的 children 列表中移除
            // remove from its parent's children list
            // 调用 graphlib.setParent，不需要更新数据
            this.setParent(id, undefined, COMBO_KEY, false);
            // 将子节点移动到父节点的 children 列表中
            // move the children to the grandparent's children list
            this.model.getChildren(id, COMBO_KEY).forEach((child) => {
                const childData = toG6Data(child);
                const childId = idOf(childData);
                this.setParent(idOf(childData), grandParentId, COMBO_KEY, false);
                const value = mergeElementsData(childData, {
                    id: idOf(childData),
                    style: { parentId: grandParentId },
                });
                this.pushChange({
                    value,
                    // @ts-ignore
                    original: childData,
                    type: this.isCombo(childId) ? ChangeType.ComboUpdated : ChangeType.NodeUpdated,
                });
                this.model.mergeNodeData(idOf(childData), value);
            });
            if (!isUndefined(grandParentId))
                this.refreshComboData(grandParentId);
        }
    }
    /**
     * <zh/> 获取元素的类型
     *
     * <en/> Get the type of the element
     * @param id - <zh/> 元素 ID | <en/> ID of the element
     * @returns <zh/> 元素类型 | <en/> type of the element
     */
    getElementType(id) {
        if (this.model.hasNode(id)) {
            if (this.isCombo(id))
                return 'combo';
            return 'node';
        }
        if (this.model.hasEdge(id))
            return 'edge';
        throw new Error(`Unknown element type of id: ${id}`);
    }
    /**
     * <zh/> 计算元素置顶后的 zIndex
     *
     * <en/> Calculate the zIndex after the element is placed on top
     * @param id - <zh/> 元素 ID | <en/> ID of the element
     * @returns <zh/> zIndex | <en/> zIndex
     */
    getFrontZIndex(id) {
        const elementType = this.getElementType(id);
        let elementsToCompare = [];
        if (elementType === 'combo') {
            const ancestors = [id, ...this.getAncestorsData(id, COMBO_KEY).map(idOf)];
            // 过滤掉以下 combo 不参与 zIndex 计算
            // - 未展开的 combo
            // - 当前 combo 及其祖先和后代
            // The following combos do not participate in zIndex calculation
            // - collapsed combo
            // - current combo and its ancestors and descendants
            elementsToCompare = this.getComboData().filter((combo) => {
                const comboId = idOf(combo);
                const comboAncestors = this.getAncestorsData(comboId, COMBO_KEY);
                const comboAncestorIds = comboAncestors.map(idOf);
                return (!comboAncestors.some((combo) => !!combo.style?.collapsed) &&
                    !ancestors.includes(comboId) &&
                    !comboAncestorIds.includes(comboId));
            });
        }
        else
            elementsToCompare = this.getNodeData().filter((node) => idOf(node) !== id);
        return Math.max(0, ...elementsToCompare.map(zIndexOf)) + 1;
    }
    destroy() {
        const { model } = this;
        const nodes = model.getAllNodes();
        const edges = model.getAllEdges();
        model.removeEdges(edges.map((edge) => edge.id));
        model.removeNodes(nodes.map((node) => node.id));
        // @ts-expect-error force delete
        this.context = {};
    }
}
//# sourceMappingURL=data.js.map