"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.executor = void 0;
const util_1 = require("@antv/util");
const constants_1 = require("../constants");
const registry_1 = require("../registry");
const animation_1 = require("../utils/animation");
/**
 * <zh/> 动画 Spec 执行器
 *
 * <en/> Animation Spec Executor
 * @param element - <zh/> 要执行动画的图形 | <en/> shape to execute animation
 * @param animation - <zh/> 动画 Spec | <en/> animation specification
 * @param commonEffectTiming - <zh/> 动画公共配置 | <en/> common animation configuration
 * @param context - <zh/> 动画执行上下文 | <en/> animation execution context
 * @returns <zh/> 动画实例 | <en/> animation instance
 */
const executor = (element, animation, commonEffectTiming, context) => {
    if (!animation)
        return null;
    const { animationsFilter = () => true } = context;
    const animations = ((0, util_1.isString)(animation) ? (0, registry_1.getExtension)('animation', animation) || [] : animation).filter(animationsFilter);
    if (animations.length === 0)
        return null;
    const { originalStyle, modifiedStyle, states } = context;
    /**
     * <zh/> 获取图形关键帧样式
     *
     * <en/> Get the keyframe style of the shape
     * @param shapeID - <zh/> 图形 ID | <en/> shape ID
     * @returns <zh/> 图形关键帧样式 | <en/> keyframe style of the shape
     */
    const getKeyframeStyle = (shapeID) => {
        if (shapeID) {
            const shape = element.getElementById(shapeID);
            const name = `get${(0, util_1.upperFirst)(shapeID)}Style`;
            const styler = element?.[name]?.bind(element) || ((attrs) => attrs);
            const fromStyle = styler?.({ ...element.attributes, ...originalStyle }) || {};
            const toStyle = { ...shape.attributes };
            return { shape, fromStyle, toStyle };
        }
        else {
            const shape = element;
            const fromStyle = originalStyle;
            const toStyle = { ...shape.attributes, ...modifiedStyle };
            return { shape, fromStyle, toStyle };
        }
    };
    let keyResult;
    const results = animations
        .map(({ fields, shape: shapeID, states: enabledStates, ...individualEffectTiming }) => {
        if (enabledStates === undefined || enabledStates?.some((state) => states.includes(state))) {
            const { shape, fromStyle, toStyle } = getKeyframeStyle(shapeID);
            if (!shape)
                return null;
            const keyframes = [{}, {}];
            fields.forEach((attr) => {
                Object.assign(keyframes[0], { [attr]: fromStyle[attr] ?? (0, animation_1.inferDefaultValue)(attr) });
                Object.assign(keyframes[1], { [attr]: toStyle[attr] ?? (0, animation_1.inferDefaultValue)(attr) });
            });
            const result = (0, animation_1.executeAnimation)(shape, (0, animation_1.preprocessKeyframes)(keyframes), {
                ...constants_1.DEFAULT_ELEMENTS_ANIMATION_OPTIONS,
                ...commonEffectTiming,
                ...individualEffectTiming,
            });
            if (shapeID === undefined)
                keyResult = result;
            return result;
        }
        return null;
    })
        .filter(Boolean);
    const mainResult = keyResult || results?.[0];
    if (!mainResult)
        return null;
    return (0, animation_1.createAnimationsProxy)(mainResult, results.filter((result) => result !== mainResult));
};
exports.executor = executor;
//# sourceMappingURL=executor.js.map