"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BrushSelect = exports.DEFAULT_STYLE = void 0;
const g_1 = require("@antv/g");
const util_1 = require("@antv/util");
const constants_1 = require("../constants");
const brush_1 = require("../utils/behaviors/brush");
const utils_1 = require("../utils/behaviors/utils");
const shortcut_1 = require("../utils/shortcut");
const base_behavior_1 = require("./base-behavior");
const SHOW_RECT_ID = 'g6-brush-select-rect-id';
exports.DEFAULT_STYLE = {
    lineWidth: 1,
    color: '#EEF6FF',
    stroke: '#DDEEFE',
    fillOpacity: 0.4,
    zIndex: 2,
};
class BrushSelect extends base_behavior_1.BaseBehavior {
    static defaultOptions = {
        enable: true,
        trigger: ['drag'],
        immediately: false,
        state: 'selected',
        mode: 'default',
        animation: false,
        enableElements: ['node', 'combo', 'edge'],
        style: {
            size: 0,
            type: 'rect',
            ...exports.DEFAULT_STYLE,
            pointerEvents: 'none',
        },
    };
    startPoint;
    endPoint;
    rectShape;
    shortcut;
    selectElementFn = brush_1.isBBoxCenterInRect;
    constructor(context, options) {
        super(context, Object.assign({}, BrushSelect.defaultOptions, options));
        this.shortcut = new shortcut_1.Shortcut(context.graph);
        if (options.type === 'lasso-select')
            return;
        this.bindEvents();
    }
    pointerDown = async (event) => {
        if (!this.validate(event) || !this.isKeydown() || this.startPoint)
            return;
        const { style, trigger } = this.options;
        const triggers = (Array.isArray(trigger) ? trigger : [trigger]);
        if (event.targetType !== 'canvas' && triggers.includes('drag'))
            return;
        const { canvas } = this.context;
        this.rectShape = new g_1.Rect({
            id: SHOW_RECT_ID,
            style: {
                ...BrushSelect.defaultOptions.style,
                fill: style.color || exports.DEFAULT_STYLE.color,
                ...style,
                pointerEvents: 'none',
            },
        });
        canvas.appendChild(this.rectShape);
        this.startPoint = [event.canvas.x, event.canvas.y];
    };
    pointerMove = async (event) => {
        if (!this.startPoint)
            return;
        const { immediately, mode } = this.options;
        this.endPoint = [event.canvas.x, event.canvas.y];
        this.rectShape?.attr({
            x: Math.min(this.endPoint[0], this.startPoint[0]),
            y: Math.min(this.endPoint[1], this.startPoint[1]),
            width: Math.abs(this.endPoint[0] - this.startPoint[0]),
            height: Math.abs(this.endPoint[1] - this.startPoint[1]),
        });
        if (immediately && mode === 'default') {
            this.updateElementState((0, brush_1.getRectPoints)(this.startPoint, this.endPoint));
        }
    };
    pointerUp = async (event) => {
        if (!this.startPoint)
            return;
        if (!this.endPoint) {
            await this.clearBrush();
            return;
        }
        const points = (0, brush_1.getRectPoints)(this.startPoint, [event.canvas.x, event.canvas.y]);
        this.updateElementState(points);
        await this.clearBrush();
    };
    clearSelected = () => {
        if (this.endPoint)
            return;
        const { graph } = this.context;
        const selects = (0, utils_1.getAllElementState)(graph, () => []);
        graph.setElementState(selects, this.options.animation);
    };
    updateElementState = (points) => {
        const { graph } = this.context;
        const { enableElements, state, mode, onSelect } = this.options;
        // 框选选中的 ids
        const rectSelectIds = this.getPointsSelectIds(graph, points, enableElements);
        // state mode 框选逻辑
        let stateChangeFn = (id, oldState) => (rectSelectIds.includes(id) ? [state] : []);
        switch (mode) {
            case 'union':
                stateChangeFn = (id, oldState) => (rectSelectIds.includes(id) ? [state] : oldState);
                break;
            case 'diff':
                stateChangeFn = (id, oldState) => {
                    if (rectSelectIds.includes(id)) {
                        return oldState.includes(state) ? [] : [state];
                    }
                    return oldState;
                };
                break;
            case 'intersect':
                stateChangeFn = (id, oldState) => {
                    if (rectSelectIds.includes(id)) {
                        return oldState.includes(state) ? [state] : [];
                    }
                    return oldState;
                };
                break;
        }
        let states = (0, utils_1.getAllElementState)(graph, stateChangeFn);
        if (enableElements.includes('edge')) {
            (0, utils_1.transformEdgeState)(graph, states, state);
        }
        if ((0, util_1.isFunction)(onSelect)) {
            states = onSelect(states);
        }
        graph.setElementState(states, this.options.animation);
    };
    clearBrush = async () => {
        this.rectShape?.remove();
        this.rectShape = undefined;
        this.startPoint = undefined;
        this.endPoint = undefined;
    };
    getPointsSelectIds = (graph, points, itemTypes) => {
        const selectedNodeIds = [];
        const selectedComboIds = [];
        if (itemTypes.includes('node')) {
            graph.getNodeData().forEach((node) => {
                const { id } = node;
                if (graph.getElementVisibility(id) !== 'hidden' && // hidden node is not selectable
                    this.selectElementFn(graph, id, points)) {
                    selectedNodeIds.push(id);
                }
            });
        }
        if (itemTypes.includes('combo')) {
            graph.getComboData().forEach((combo) => {
                const { id } = combo;
                if (graph.getElementVisibility(id) !== 'hidden' && // hidden combo is not selectable
                    this.selectElementFn(graph, id, points)) {
                    selectedComboIds.push(id);
                }
            });
        }
        return [...selectedNodeIds, ...selectedComboIds];
    };
    // 当前按键是否和 trigger 配置一致
    isKeydown() {
        const { trigger } = this.options;
        const keys = (Array.isArray(trigger) ? trigger : [trigger]);
        if (keys.length === 0 || keys.includes('drag'))
            return true;
        return this.shortcut?.match(keys);
    }
    validate(event) {
        if (this.destroyed)
            return false;
        const { enable } = this.options;
        if ((0, util_1.isFunction)(enable))
            return enable(event);
        return !!enable;
    }
    bindEvents() {
        const { graph } = this.context;
        this.unbindEvents();
        graph.on(constants_1.CommonEvent.POINTER_DOWN, this.pointerDown);
        graph.on(constants_1.CommonEvent.POINTER_MOVE, this.pointerMove);
        graph.on(constants_1.CommonEvent.POINTER_UP, this.pointerUp);
        graph.on(`canvas:${constants_1.CommonEvent.CLICK}`, this.clearSelected);
    }
    unbindEvents() {
        const { graph } = this.context;
        graph.off(constants_1.CommonEvent.POINTER_DOWN, this.pointerDown);
        graph.off(constants_1.CommonEvent.POINTER_MOVE, this.pointerMove);
        graph.off(constants_1.CommonEvent.POINTER_UP, this.pointerUp);
        graph.off(`canvas:${constants_1.CommonEvent.CLICK}`, this.clearSelected);
    }
    destroy() {
        this.unbindEvents();
        super.destroy();
    }
}
exports.BrushSelect = BrushSelect;
//# sourceMappingURL=brush-select.js.map