"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClickElement = void 0;
const util_1 = require("@antv/util");
const constants_1 = require("../constants");
const element_1 = require("../constants/element");
const id_1 = require("../utils/id");
const relation_1 = require("../utils/relation");
const shortcut_1 = require("../utils/shortcut");
const base_behavior_1 = require("./base-behavior");
class ClickElement extends base_behavior_1.BaseBehavior {
    selectedElementIds = [];
    shortcut;
    static defaultOptions = {
        animation: true,
        enable: true,
        multiple: false,
        trigger: ['shift'],
        selectedState: 'selected',
        unselectedState: undefined,
        degree: 0,
    };
    constructor(context, options) {
        super(context, Object.assign({}, ClickElement.defaultOptions, options));
        this.shortcut = new shortcut_1.Shortcut(context.graph);
        this.bindEvents();
    }
    bindEvents() {
        const { graph } = this.context;
        this.unbindEvents();
        element_1.ELEMENT_TYPES.forEach((type) => {
            graph.on(`${type}:${constants_1.CommonEvent.CLICK}`, this.onClickElement);
        });
        graph.on(`canvas:${constants_1.CommonEvent.CLICK}`, this.onClickCanvas);
    }
    onClickElement = (event) => {
        if (!this.validate(event))
            return;
        this.updateElementsState(event, false);
        this.updateElementsState(event, true);
    };
    onClickCanvas = (event) => {
        if (!this.validate(event))
            return;
        this.updateElementsState(event, false);
        this.selectedElementIds = [];
    };
    updateElementsState = (event, add) => {
        if (!this.options.selectedState && !this.options.unselectedState)
            return;
        const { graph } = this.context;
        const { targetType, target } = event;
        if (add) {
            // 如果当前元素已经被选中，则取消选中 | If the current element is already selected, deselect it
            if (this.selectedElementIds.includes(target.id)) {
                this.selectedElementIds = this.selectedElementIds.filter((id) => id !== target.id);
            }
            else {
                const selectedElementIds = (0, relation_1.getElementNthDegreeIds)(graph, targetType, target.id, this.options.degree);
                const isMultiple = this.options.multiple && this.shortcut.match(this.options.trigger);
                if (!isMultiple) {
                    this.selectedElementIds = selectedElementIds;
                }
                else {
                    this.selectedElementIds.push(...selectedElementIds);
                }
            }
        }
        if (!this.selectedElementIds.length)
            return;
        const states = {};
        if (this.options.selectedState) {
            Object.assign(states, this.getElementsState(this.selectedElementIds, this.options.selectedState, add));
        }
        if (this.options.unselectedState) {
            const inactiveIds = (0, id_1.idsOf)(graph.getData(), true).filter((id) => !this.selectedElementIds.includes(id));
            Object.assign(states, this.getElementsState(inactiveIds, this.options.unselectedState, add));
        }
        graph.setElementState(states, this.options.animation);
    };
    getElementsState = (ids, state, add) => {
        const { graph } = this.context;
        const states = {};
        ids.forEach((id) => {
            const currentState = graph.getElementState(id);
            const updatedState = add ? [...currentState, state] : currentState.filter((s) => s !== state);
            states[id] = updatedState;
        });
        return states;
    };
    validate(event) {
        if (this.destroyed)
            return false;
        const { enable } = this.options;
        if ((0, util_1.isFunction)(enable))
            return enable(event);
        return !!enable;
    }
    unbindEvents() {
        const { graph } = this.context;
        element_1.ELEMENT_TYPES.forEach((type) => {
            graph.off(`${type}:${constants_1.CommonEvent.CLICK}`, this.onClickElement);
        });
        graph.off(`canvas:${constants_1.CommonEvent.CLICK}`, this.onClickCanvas);
    }
    destroy() {
        this.unbindEvents();
        super.destroy();
    }
}
exports.ClickElement = ClickElement;
//# sourceMappingURL=click-element.js.map