"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Contextmenu = void 0;
const contextmenu_1 = require("../utils/contextmenu");
const dom_1 = require("../utils/dom");
const base_plugin_1 = require("./base-plugin");
/**
 * <zh/> 支持处理事件，并显示右键菜单，在菜单点击之后，可以触发相应的事件。
 * <en/> Support processing events and displaying right-click menus.
 *       After clicking the menu, you can trigger the corresponding event.
 */
class Contextmenu extends base_plugin_1.BasePlugin {
    static defaultOptions = {
        trigger: 'contextmenu',
        offset: [4, 4],
        loadingContent: '<div class="g6-contextmenu-loading">Loading...</div>',
        getContent: () => 'It is a empty context menu.',
        enable: () => true,
    };
    $element = (0, dom_1.createPluginContainer)('contextmenu', false);
    constructor(context, options) {
        super(context, Object.assign({}, Contextmenu.defaultOptions, options));
        const $container = this.context.canvas.getContainer();
        $container.appendChild(this.$element);
        // 设置样式
        (0, dom_1.insertDOM)('g6-contextmenu-css', 'style', {}, contextmenu_1.CONTEXTMENU_CSS, document.head);
        this.update(options);
    }
    /**
     * <zh/> 根据传入的元素，显示右键菜单。
     * <en/> Display the right-click menu based on the incoming element.
     * @param e - 事件
     */
    async showContextmenu(e) {
        const { enable, offset } = this.options;
        if ((typeof enable === 'function' && !enable(e)) || !enable) {
            this.hideContextmenu();
            return;
        }
        const content = await this.getDOMContent(e);
        if (content instanceof HTMLElement) {
            this.$element.appendChild(content);
        }
        else {
            this.$element.innerHTML = content;
        }
        // NOTICE: 为什么事件中的 client 是相对浏览器，而不是画布容器？
        const clientRect = this.context.graph.getCanvas().getContainer().getBoundingClientRect();
        this.$element.style.left = `${e.client.x - clientRect.left + offset[0]}px`;
        this.$element.style.top = `${e.client.y - clientRect.top + offset[1]}px`;
        this.$element.style.display = 'block';
    }
    /**
     * <zh/> 隐藏右键菜单。
     * <en/> Hide the right-click menu.
     */
    hideContextmenu() {
        this.$element.style.display = 'none';
    }
    /**
     * <zh/> 更新右键菜单的配置项。
     * <en/> Update the configuration of the right-click menu.
     * @param options - 配置项
     */
    update(options) {
        this.unbindEvents();
        super.update(options);
        this.bindEvents();
    }
    /**
     * <zh/> 销毁右键菜单。
     * <en/> Destroy the right-click menu.
     */
    destroy() {
        this.unbindEvents();
        super.destroy();
        this.$element.remove();
    }
    async getDOMContent(e) {
        const { getContent, getItems } = this.options;
        if (getItems) {
            return (0, contextmenu_1.getContentFromItems)(await getItems(e));
        }
        return await getContent(e);
    }
    bindEvents() {
        const { graph } = this.context;
        const { trigger } = this.options;
        graph.on(`canvas:${trigger}`, this.onTriggerEvent);
        graph.on(`node:${trigger}`, this.onTriggerEvent);
        graph.on(`edge:${trigger}`, this.onTriggerEvent);
        graph.on(`combo:${trigger}`, this.onTriggerEvent);
        document.addEventListener('click', this.onMenuItemClick);
    }
    unbindEvents() {
        const { graph } = this.context;
        const { trigger } = this.options;
        graph.off(`canvas:${trigger}`, this.onTriggerEvent);
        graph.off(`node:${trigger}`, this.onTriggerEvent);
        graph.off(`edge:${trigger}`, this.onTriggerEvent);
        graph.off(`combo:${trigger}`, this.onTriggerEvent);
        document.removeEventListener('click', this.onMenuItemClick);
    }
    onTriggerEvent = (e) => {
        // `contextmenu` 事件默认会触发浏览器的右键菜单，需要阻止默认事件
        // `click` 事件不需要阻止默认事件
        e.preventDefault?.();
        this.showContextmenu(e);
    };
    onMenuItemClick = (e) => {
        const { onClick } = this.options;
        if (e.target instanceof HTMLElement) {
            if (e.target.className.includes('g6-contextmenu-li')) {
                const v = e.target.getAttribute('value');
                onClick && onClick(v, e.target);
                this.hideContextmenu();
            }
            // 点击其他地方，隐藏菜单
            if (!this.context.graph.getCanvas().getContainer().contains(e.target)) {
                this.hideContextmenu();
            }
        }
    };
}
exports.Contextmenu = Contextmenu;
//# sourceMappingURL=contextmenu.js.map