"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Hull = void 0;
const util_1 = require("@antv/util");
const hull_js_1 = __importDefault(require("hull.js"));
const constants_1 = require("../../constants");
const id_1 = require("../../utils/id");
const position_1 = require("../../utils/position");
const base_plugin_1 = require("../base-plugin");
const shape_1 = require("./shape");
const util_2 = require("./util");
class Hull extends base_plugin_1.BasePlugin {
    shape;
    /**
     * <zh/> 在 Hull 上的元素
     * <en/> Element Ids on Hull
     */
    hullMemberIds = [];
    /**
     * <zh/> Hull 绘制路径
     * <en/> Hull path
     */
    path;
    /**
     * <zh> 是否初次渲染完成
     * <en> Whether the first rendering is completed
     */
    firstRender = false;
    static defaultOptions = {
        members: [],
        padding: 10,
        corner: 'rounded',
        concavity: Infinity,
        /** hull style */
        fill: 'lightblue',
        stroke: 'blue',
        opacity: 0.2,
    };
    constructor(context, options) {
        super(context, Object.assign({}, Hull.defaultOptions, options));
        this.bindEvents();
    }
    bindEvents() {
        this.context.graph.on(constants_1.GraphEvent.AFTER_RENDER, this.drawHull);
        this.context.graph.on(constants_1.GraphEvent.AFTER_ELEMENT_UPDATE, this.updateHullPath);
    }
    getHullStyle(forceUpdate) {
        const { members, padding, corner, ...style } = this.options;
        return { ...style, path: this.getHullPath(forceUpdate) };
    }
    drawHull = () => {
        this.shape = new shape_1.Hull({ style: this.getHullStyle() });
        this.context.canvas.appendChild(this.shape);
        this.firstRender = true;
    };
    updateHullPath = (event) => {
        if (!this.firstRender)
            return;
        if (!this.options.members.includes((0, id_1.idOf)(event.data)))
            return;
        this.shape.update({ path: this.getHullPath(true) });
    };
    getHullPath = (forceUpdate = false) => {
        const { graph } = this.context;
        const data = this.options.members.map((id) => graph.getNodeData(id));
        const vertices = (0, hull_js_1.default)(data.map(position_1.positionOf), this.options.concavity).slice(1).reverse();
        const hullMemberIds = vertices.flatMap((point) => data.filter((m) => (0, util_1.isEqual)((0, position_1.positionOf)(m), point)).map(id_1.idOf));
        if ((0, util_1.isEqual)(hullMemberIds, this.hullMemberIds) && !forceUpdate)
            return this.path;
        this.hullMemberIds = hullMemberIds;
        this.path = (0, util_2.computeHullPath)(vertices, this.getPadding(), this.options.corner);
        return this.path;
    };
    getPadding() {
        const { graph } = this.context;
        const memberPadding = this.hullMemberIds.reduce((acc, id) => {
            const { halfExtents } = graph.getElementRenderBounds(id);
            const size = Math.max(halfExtents[0], halfExtents[1]);
            return Math.max(acc, size);
        }, 0);
        return memberPadding + this.options.padding;
    }
    addMembers(members) {
        const membersToAdd = Array.isArray(members) ? members : [members];
        this.options.members = [...new Set([...this.options.members, ...membersToAdd])];
        this.shape.update({ path: this.getHullPath() });
    }
    removeMembers(members) {
        const membersToRemove = Array.isArray(members) ? members : [members];
        this.options.members = this.options.members.filter((id) => !membersToRemove.includes(id));
        if (membersToRemove.some((id) => this.hullMemberIds.includes(id))) {
            this.shape.update({ path: this.getHullPath() });
        }
    }
    updateMembers(members) {
        this.options.members = (0, util_1.isFunction)(members) ? members(this.options.members) : members;
        this.shape.update(this.getHullStyle(true));
    }
    updateOptions(options) {
        this.options = ((0, util_1.isFunction)(options) ? options(this.options) : options);
        this.shape.update({ ...this.options, path: this.getHullPath(true) });
    }
    destroy() {
        this.context.graph.off(constants_1.GraphEvent.AFTER_DRAW, this.drawHull);
        this.shape.destroy();
        this.hullMemberIds = [];
        super.destroy();
    }
}
exports.Hull = Hull;
//# sourceMappingURL=index.js.map