"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Canvas = void 0;
const g_1 = require("@antv/g");
const g_canvas_1 = require("@antv/g-canvas");
const g_plugin_dragndrop_1 = require("@antv/g-plugin-dragndrop");
const util_1 = require("@antv/util");
const bbox_1 = require("../utils/bbox");
/**
 * @deprecated this canvas will be replace by layered canvas
 */
class Canvas {
    config;
    background;
    main;
    label;
    transient;
    get canvas() {
        return {
            main: this.main,
            label: this.label,
            transient: this.transient,
            background: this.background,
        };
    }
    get document() {
        return this.main.document;
    }
    renderers;
    initialized = false;
    constructor(config) {
        this.config = config;
    }
    async init() {
        if (this.initialized)
            return;
        const { renderer: getRenderer, background, ...restConfig } = this.config;
        const names = ['main', 'label', 'transient', 'background'];
        const { renderers, canvas } = names.reduce((acc, name) => {
            const renderer = (0, util_1.isFunction)(getRenderer) ? getRenderer?.(name) : new g_canvas_1.Renderer();
            if (name === 'main') {
                renderer.registerPlugin(new g_plugin_dragndrop_1.Plugin({
                    isDocumentDraggable: true,
                    isDocumentDroppable: true,
                    dragstartDistanceThreshold: 10,
                    dragstartTimeThreshold: 100,
                }));
            }
            else {
                renderer.unregisterPlugin(renderer.getPlugin('dom-interaction'));
            }
            const canvas = new g_1.Canvas({
                renderer,
                supportsMutipleCanvasesInOneContainer: true,
                ...restConfig,
            });
            acc.renderers[name] = renderer;
            acc.canvas[name] = canvas;
            this[name] = canvas;
            return acc;
        }, { renderers: {}, canvas: {} });
        this.renderers = renderers;
        Object.entries(canvas).forEach(([name, canvas]) => {
            const domElement = canvas.getContextService().getDomElement();
            domElement.style.position = 'absolute';
            domElement.style.outline = 'none';
            domElement.tabIndex = 1;
            if (name !== 'main')
                domElement.style.pointerEvents = 'none';
        });
        this.setBackground();
        await Promise.all(Object.values(this.canvas).map((canvas) => canvas.ready));
        this.initialized = true;
    }
    getRendererType(layer = 'main') {
        const plugins = this.renderers[layer].getPlugins();
        for (const plugin of plugins) {
            if (plugin.name === 'canvas-renderer')
                return 'canvas';
            if (plugin.name === 'svg-renderer')
                return 'svg';
            if (plugin.name === 'device-renderer')
                return 'gpu';
        }
        return 'unknown';
    }
    get context() {
        return this.main.context;
    }
    getDevice() {
        // @ts-expect-error deviceRendererPlugin is private
        return this.main.context?.deviceRendererPlugin?.getDevice();
    }
    getConfig() {
        return this.config;
    }
    setBackground(background = this.config.background) {
        const container = this.getContainer();
        if (container && background) {
            container.style.background = background;
            container.style.transition = 'background 0.5s';
        }
    }
    setCursor(cursor) {
        Object.values(this.canvas).forEach((canvas) => {
            canvas.setCursor(cursor);
        });
    }
    getSize() {
        return [this.config.width || 0, this.config.height || 0];
    }
    resize(width, height) {
        this.config.width = width;
        this.config.height = height;
        Object.values(this.canvas).forEach((canvas) => {
            canvas.resize(width, height);
        });
    }
    getCamera() {
        return this.main.getCamera();
    }
    getBounds() {
        return (0, bbox_1.getCombinedBBox)(Object.values(this.canvas)
            .map((canvas) => canvas.document.documentElement)
            .filter((el) => el.childNodes.length > 0)
            .map((el) => el.getRenderBounds()));
    }
    getContainer() {
        const container = this.config.container;
        return (0, util_1.isString)(container) ? document.getElementById(container) : container;
    }
    appendChild(child) {
        const layer = (child.style?.$layer || 'main');
        return this[layer].appendChild(child);
    }
    getContextService() {
        return this.main.getContextService();
    }
    viewport2Client(viewport) {
        return this.main.viewport2Client(viewport);
    }
    viewport2Canvas(viewport) {
        return this.main.viewport2Canvas(viewport);
    }
    client2Viewport(client) {
        return this.main.client2Viewport(client);
    }
    canvas2Viewport(canvas) {
        return this.main.canvas2Viewport(canvas);
    }
    async toDataURL(options = {}) {
        const devicePixelRatio = window.devicePixelRatio || 1;
        const { mode = 'viewport', ...restOptions } = options;
        let startX = 0;
        let startY = 0;
        let width = 0;
        let height = 0;
        if (mode === 'viewport') {
            [width, height] = [this.config.width || 0, this.config.height || 0];
        }
        else if (mode === 'overall') {
            const bounds = this.getBounds();
            const size = (0, bbox_1.getBBoxSize)(bounds);
            [startX, startY] = bounds.min;
            [width, height] = size;
        }
        const container = (0, util_1.createDOM)('<div id="virtual-image"></div>');
        const offscreenCanvas = new g_1.Canvas({
            width,
            height,
            renderer: new g_canvas_1.Renderer(),
            devicePixelRatio,
            container,
            background: this.background.getConfig().background,
        });
        await offscreenCanvas.ready;
        offscreenCanvas.appendChild(this.background.getRoot().cloneNode(true));
        offscreenCanvas.appendChild(this.main.getRoot().cloneNode(true));
        // Handle label canvas
        const label = this.label.getRoot().cloneNode(true);
        const originCanvasPosition = offscreenCanvas.viewport2Canvas({ x: 0, y: 0 });
        const currentCanvasPosition = this.main.viewport2Canvas({ x: 0, y: 0 });
        label.translate([
            currentCanvasPosition.x - originCanvasPosition.x,
            currentCanvasPosition.y - originCanvasPosition.y,
        ]);
        label.scale(1 / this.main.getCamera().getZoom());
        offscreenCanvas.appendChild(label);
        offscreenCanvas.appendChild(this.transient.getRoot().cloneNode(true));
        const camera = this.main.getCamera();
        const offscreenCamera = offscreenCanvas.getCamera();
        if (mode === 'viewport') {
            offscreenCamera.setZoom(camera.getZoom());
            offscreenCamera.setPosition(camera.getPosition());
            offscreenCamera.setFocalPoint(camera.getFocalPoint());
        }
        else if (mode === 'overall') {
            const [x, y, z] = offscreenCamera.getPosition();
            const [fx, fy, fz] = offscreenCamera.getFocalPoint();
            offscreenCamera.setPosition([x + startX, y + startY, z]);
            offscreenCamera.setFocalPoint([fx + startX, fy + startY, fz]);
        }
        const contextService = offscreenCanvas.getContextService();
        return new Promise((resolve) => {
            offscreenCanvas.on(g_1.CanvasEvent.AFTER_RENDER, async () => {
                const url = await contextService.toDataURL(restOptions);
                resolve(url);
            });
        });
    }
    destroy() {
        this.config = {};
        // @ts-expect-error force delete
        this.renderers = {};
        Object.entries(this.canvas).forEach(([name, canvas]) => {
            const camera = canvas.getCamera();
            // @ts-expect-error landmark is private
            if (camera.landmarks?.length) {
                camera.cancelLandmarkAnimation();
            }
            destroyCanvas(canvas);
            // @ts-expect-error force delete
            this[name] = undefined;
        });
    }
}
exports.Canvas = Canvas;
/**
 * <zh/> G Canvas destroy 未处理动画对象，导致内存泄漏
 *
 * <en/> G Canvas destroy does not handle animation objects, causing memory leaks
 * @param canvas GCanvas
 * @description
 * <zh/> 这些操作都应该在 G 中完成，这里只是一个临时的解决方案
 * 此操作大概能在测试环节降低 10～20% 的内存占用（从 2800MB 降低到 2200MB）
 *
 * <en/> These operations should be completed in G, this is just a temporary solution
 * This operation can reduce memory usage by 10-20% in the test environment (from 2800MB to 2200MB)
 */
function destroyCanvas(canvas) {
    canvas.destroy();
    // 移除相机事件 / Remove camera events
    const camera = canvas.getCamera();
    camera.eventEmitter.removeAllListeners();
    canvas.document.timeline.destroy();
    // @ts-expect-error private property
    const { animationsWithPromises } = canvas.document.timeline;
    // 释放 target 对象图形 / Release target object graphics
    animationsWithPromises.forEach((animation) => {
        if (animation.effect.target)
            animation.effect.target = null;
        // @ts-expect-error private property
        if (animation.effect.computedTiming)
            animation.effect.computedTiming = null;
    });
    // @ts-expect-error private property
    canvas.document.timeline.animationsWithPromises = [];
    // @ts-expect-error private property
    canvas.document.timeline.rafCallbacks = [];
    // @ts-expect-error private property
    canvas.document.timeline = null;
}
//# sourceMappingURL=canvas.js.map