"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ElementController = void 0;
const g_1 = require("@antv/g");
const util_1 = require("@antv/util");
const animations_1 = require("../animations");
const constants_1 = require("../constants");
const element_1 = require("../constants/element");
const registry_1 = require("../registry");
const animation_1 = require("../utils/animation");
const cache_1 = require("../utils/cache");
const change_1 = require("../utils/change");
const element_2 = require("../utils/element");
const event_1 = require("../utils/event");
const id_1 = require("../utils/id");
const palette_1 = require("../utils/palette");
const style_1 = require("../utils/style");
class ElementController {
    context;
    container;
    elementMap = {};
    shapeTypeMap = {};
    constructor(context) {
        this.context = context;
    }
    init() {
        if (!this.container) {
            const { canvas } = this.context;
            this.container = {
                node: canvas.appendChild(new g_1.Group({ style: { zIndex: 2 } })),
                edge: canvas.appendChild(new g_1.Group({ style: { zIndex: 1 } })),
                combo: canvas.appendChild(new g_1.Group({ style: { zIndex: 0 } })),
            };
        }
    }
    emit(event) {
        (0, event_1.emit)(this.context.graph, event);
    }
    forEachElementData(callback) {
        element_1.ELEMENT_TYPES.forEach((elementType) => {
            const elementData = this.context.model.getElementData(elementType);
            callback(elementType, elementData);
        });
    }
    getTheme(elementType) {
        const { theme } = this.context.options;
        if (!theme)
            return {};
        const themeConfig = (0, registry_1.getExtension)('theme', theme);
        return themeConfig?.[elementType] || {};
    }
    getThemeStyle(elementType) {
        return this.getTheme(elementType).style || {};
    }
    getThemeStateStyle(elementType, states) {
        const { state = {} } = this.getTheme(elementType);
        return Object.assign({}, ...states.map((name) => state[name] || {}));
    }
    paletteStyle = {};
    computePaletteStyle() {
        const { options } = this.context;
        this.paletteStyle = {};
        this.forEachElementData((elementType, elementData) => {
            const palette = Object.assign({}, (0, palette_1.parsePalette)(this.getTheme(elementType)?.palette), (0, palette_1.parsePalette)(options[elementType]?.palette));
            if (!(0, util_1.isEmpty)(palette) && palette?.field) {
                Object.assign(this.paletteStyle, (0, palette_1.assignColorByPalette)(elementData, palette));
            }
        });
    }
    getPaletteStyle(id) {
        const color = this.paletteStyle[id];
        if (!color)
            return {};
        return { color };
    }
    defaultStyle = {};
    /**
     * <zh/> 计算单个元素的默认样式
     *
     * <en/> compute default style of single element
     */
    computedElementDefaultStyle(elementType, context) {
        const { options } = this.context;
        const defaultStyle = options[elementType]?.style || {};
        this.defaultStyle[(0, id_1.idOf)(context.datum)] = (0, style_1.computeElementCallbackStyle)(defaultStyle, context);
    }
    computeElementsDefaultStyle(ids) {
        this.forEachElementData((elementType, elementData) => {
            elementData
                .filter((datum) => ids === undefined || ids.includes((0, id_1.idOf)(datum)))
                .forEach((datum, index) => {
                this.computedElementDefaultStyle(elementType, { datum, index, elementData });
            });
        });
    }
    getDefaultStyle(id) {
        return this.defaultStyle[id] || {};
    }
    getElementState(id) {
        try {
            const { model } = this.context;
            return model.getElementState(id);
        }
        catch {
            return [];
        }
    }
    stateStyle = {};
    /**
     * <zh/> 获取单个元素的单个状态的样式
     *
     * <en/> get single state style of single element
     */
    getElementStateStyle(elementType, state, context) {
        const { options } = this.context;
        const stateStyle = options[elementType]?.state?.[state] || {};
        return (0, style_1.computeElementCallbackStyle)(stateStyle, context);
    }
    /**
     * <zh/> 计算单个元素的合并状态样式
     *
     * <en/> compute merged state style of single element
     */
    computeElementStatesStyle(elementType, states, context) {
        this.stateStyle[(0, id_1.idOf)(context.datum)] = Object.assign({}, ...states.map((state) => this.getElementStateStyle(elementType, state, context)));
    }
    /**
     * <zh/> 计算全部元素的状态样式
     *
     * <en/> compute state style of all elements
     * @param ids - <zh/> 计算指定元素的状态样式 | <en/> compute state style of specified elements
     */
    computeElementsStatesStyle(ids) {
        this.forEachElementData((elementType, elementData) => {
            elementData
                .filter((datum) => ids === undefined || ids.includes((0, id_1.idOf)(datum)))
                .forEach((datum, index) => {
                const states = this.getElementState((0, id_1.idOf)(datum));
                this.computeElementStatesStyle(elementType, states, { datum, index, elementData });
            });
        });
    }
    getStateStyle(id) {
        return this.stateStyle[id] || {};
    }
    computeStyle(ids) {
        this.computePaletteStyle();
        this.computeElementsDefaultStyle(ids);
        this.computeElementsStatesStyle(ids);
    }
    getElement(id) {
        return this.elementMap[id];
    }
    getNodes() {
        return this.container.node.children;
    }
    getEdges() {
        return this.container.edge.children;
    }
    getCombos() {
        return this.container.combo.children;
    }
    getAnimation(elementType, stage) {
        const { options } = this.context;
        const userDefined = options?.[elementType]?.animation;
        if (userDefined === false)
            return false;
        const userDefinedStage = userDefined?.[stage];
        if (userDefinedStage)
            return userDefinedStage;
        const themeDefined = this.getTheme(elementType)?.animation;
        if (themeDefined === false)
            return false;
        const themeDefinedStage = themeDefined?.[stage];
        return themeDefinedStage ?? false;
    }
    getAnimationExecutor(elementType, stage, animation = true, context) {
        const { options } = this.context;
        if (options.animation === false || !animation)
            return () => null;
        return (id, shape, originalStyle, modifiedStyle) => {
            return (0, animations_1.executor)(shape, this.getAnimation(elementType, stage), {}, {
                originalStyle,
                modifiedStyle,
                states: this.getElementState(id),
                ...context,
            });
        };
    }
    /**
     * <zh/> 获取边端点连接上下文
     *
     * <en/> Get edge end context
     * @param id - <zh/> 边 id | <en/> edge id
     * @returns <zh/> 边端点连接上下文 | <en/> edge end context
     * @description
     * <zh/> 只提供了最基本的节点示例和连接点位置信息，更多的上下文信息需要在边元素中计算
     *
     * <en/> Only the most basic node instances and connection point position information are provided, and more context information needs to be calculated in the edge element
     */
    getEdgeEndsContext(datum) {
        const { source, target } = datum;
        const sourceNode = this.getElement(source);
        const targetNode = this.getElement(target);
        return { sourceNode, targetNode };
    }
    getElementComputedStyle(elementType, datum) {
        const id = (0, id_1.idOf)(datum);
        // 优先级(从低到高) Priority (from low to high):
        const themeStyle = this.getThemeStyle(elementType);
        const paletteStyle = this.getPaletteStyle(id);
        const dataStyle = datum.style || {};
        const defaultStyle = this.getDefaultStyle(id);
        const themeStateStyle = this.getThemeStateStyle(elementType, this.getElementState(id));
        const stateStyle = this.getStateStyle(id);
        const style = Object.assign({}, themeStyle, paletteStyle, dataStyle, defaultStyle, themeStateStyle, stateStyle);
        if (elementType === 'edge') {
            Object.assign(style, this.getEdgeEndsContext(datum));
        }
        else if (elementType === 'combo') {
            const childrenData = this.context.model.getChildrenData(id);
            const isCollapsed = !!style.collapsed;
            const childrenNode = isCollapsed
                ? []
                : childrenData.map((child) => this.getElement((0, id_1.idOf)(child))).filter(Boolean);
            Object.assign(style, { childrenNode, childrenData });
        }
        if (!style.type) {
            style.type = { node: 'circle', edge: 'line', combo: 'circle' }[elementType];
        }
        return style;
    }
    /**
     * <zh/> 开始绘制流程
     *
     * <en/> start render process
     */
    async draw(drawContext = { animation: true }) {
        this.init();
        const data = this.computeChangesAndDrawData();
        if (!data)
            return;
        const { dataChanges, drawData } = data;
        // 计算样式 / Calculate style
        this.computeStyle();
        // 创建渲染任务 / Create render task
        const { add, update, remove } = drawData;
        const destroyTasks = this.getDestroyTasks(remove, drawContext);
        const createTasks = this.getCreateTasks(add, drawContext);
        const updateTasks = this.getUpdateTasks(update, drawContext);
        await (0, animation_1.executeAnimatableTasks)([...destroyTasks, ...createTasks, ...updateTasks], drawContext.silence
            ? {}
            : {
                before: () => this.emit(new event_1.GraphLifeCycleEvent(constants_1.GraphEvent.BEFORE_DRAW, { dataChanges, animation: drawContext.animation })),
                beforeAnimate: (animation) => this.emit(new event_1.AnimateEvent(constants_1.GraphEvent.BEFORE_ANIMATE, constants_1.AnimationType.DRAW, animation, drawData)),
                afterAnimate: (animation) => this.emit(new event_1.AnimateEvent(constants_1.GraphEvent.AFTER_ANIMATE, constants_1.AnimationType.DRAW, animation, drawData)),
                after: () => this.emit(new event_1.GraphLifeCycleEvent(constants_1.GraphEvent.AFTER_DRAW, { dataChanges, animation: drawContext.animation })),
            })?.finished;
    }
    computeChangesAndDrawData() {
        const { model } = this.context;
        const dataChanges = model.getChanges();
        const tasks = (0, change_1.reduceDataChanges)(dataChanges);
        if (tasks.length === 0)
            return null;
        const { NodeAdded = [], NodeUpdated = [], NodeRemoved = [], EdgeAdded = [], EdgeUpdated = [], EdgeRemoved = [], ComboAdded = [], ComboUpdated = [], ComboRemoved = [], } = (0, util_1.groupBy)(tasks, (change) => change.type);
        const dataOf = (data) => new Map(data.map((datum) => {
            const data = datum.value;
            return [(0, id_1.idOf)(data), data];
        }));
        const input = {
            add: {
                nodes: dataOf(NodeAdded),
                edges: dataOf(EdgeAdded),
                combos: dataOf(ComboAdded),
            },
            update: {
                nodes: dataOf(NodeUpdated),
                edges: dataOf(EdgeUpdated),
                combos: dataOf(ComboUpdated),
            },
            remove: {
                nodes: dataOf(NodeRemoved),
                edges: dataOf(EdgeRemoved),
                combos: dataOf(ComboRemoved),
            },
        };
        const drawData = this.transformData(input);
        // 清空变更 / Clear changes
        model.clearChanges();
        return { dataChanges, drawData };
    }
    transformData(input) {
        const transforms = this.context.transform.getTransformInstance();
        return Object.values(transforms).reduce((data, transform) => transform.beforeDraw(data), input);
    }
    createElement(elementType, datum, context) {
        const { animator } = context;
        const id = (0, id_1.idOf)(datum);
        const currentShape = this.getElement(id);
        if (currentShape)
            return () => null;
        const { type, ...style } = this.getElementComputedStyle(elementType, datum);
        // get shape constructor
        const Ctor = (0, registry_1.getExtension)(elementType, type);
        if (!Ctor)
            return () => null;
        this.emit(new event_1.ElementLifeCycleEvent(constants_1.GraphEvent.BEFORE_ELEMENT_CREATE, elementType, datum));
        const shape = this.container[elementType].appendChild(new Ctor({
            id,
            style: {
                context: this.context,
                ...style,
            },
        }));
        this.shapeTypeMap[id] = type;
        this.elementMap[id] = shape;
        return () => (0, animation_1.withAnimationCallbacks)(animator?.(id, shape, { ...shape.attributes, opacity: 0 }) || null, {
            after: () => {
                this.emit(new event_1.ElementLifeCycleEvent(constants_1.GraphEvent.AFTER_ELEMENT_CREATE, elementType, datum));
                shape.onCreate();
            },
        });
    }
    getCreateTasks(data, context) {
        const { nodes, edges, combos } = data;
        const iteration = [
            ['node', nodes],
            ['combo', combos],
            ['edge', edges],
        ];
        const tasks = [];
        iteration.forEach(([elementType, elementData]) => {
            if (elementData.size === 0)
                return;
            const animator = this.getAnimationExecutor(elementType, 'enter', context.animation);
            elementData.forEach((datum) => tasks.push(() => this.createElement(elementType, datum, { ...context, animator })));
        });
        return tasks;
    }
    /**
     * <zh/> 由于 show 和 hide 的时序存在差异
     * - show： 立即将元素显示出来，然后执行动画
     * - hide： 先执行动画，然后隐藏元素
     *
     * 如果在调用 hide 后立即调用 show，hide 动画完成后会将元素隐藏
     *
     * 因此需要缓存元素最新的可见性
     *
     * <en/> Due to the difference in the timing of show and hide
     * - show: immediately shows the element and then executes the animation
     * - hide: executes the animation first, and then hides the element
     *
     * If show is called immediately after hide,it hide the element after hide animation completed
     *
     * Therefore, the latest visibility of the element needs to be cached
     */
    latestElementVisibilityMap = new WeakMap();
    updateElement(elementType, datum, context) {
        const { animator } = context;
        const id = (0, id_1.idOf)(datum);
        const shape = this.getElement(id);
        if (!shape)
            return () => null;
        this.emit(new event_1.ElementLifeCycleEvent(constants_1.GraphEvent.BEFORE_ELEMENT_UPDATE, elementType, datum));
        const afterUpdate = () => {
            this.emit(new event_1.ElementLifeCycleEvent(constants_1.GraphEvent.AFTER_ELEMENT_UPDATE, elementType, datum));
            shape.onUpdate();
        };
        const { type, ...style } = this.getElementComputedStyle(elementType, datum);
        // 如果类型不同，需要先销毁原有元素，再创建新元素
        // If the type is different, you need to destroy the original element first, and then create a new element
        if (this.shapeTypeMap[id] !== type) {
            return () => {
                this.destroyElement(elementType, datum, { ...context, animation: false })();
                this.createElement(elementType, datum, { ...context, animation: false })();
                afterUpdate();
                return null;
            };
        }
        // 如果是可见性更新 / If it is a visibility update
        if (context.stage === 'visibility' && 'visibility' in style) {
            // 缓存原始透明度 / Cache original opacity
            if (!(0, cache_1.hasCachedStyle)(shape, 'opacity'))
                (0, cache_1.cacheStyle)(shape, 'opacity');
            const originalOpacity = (0, cache_1.getCachedStyle)(shape, 'opacity') ?? (0, animation_1.inferDefaultValue)('opacity');
            this.latestElementVisibilityMap.set(shape, style.visibility);
            // show
            if (style.visibility !== 'hidden') {
                (0, element_2.updateStyle)(shape, { visibility: 'visible' });
                return () => (0, animation_1.withAnimationCallbacks)(animator?.(id, shape, { ...shape.attributes, opacity: 0 }, { opacity: originalOpacity }) || null, { after: afterUpdate });
            }
            // hide
            else if (style.visibility === 'hidden') {
                return () => (0, animation_1.withAnimationCallbacks)(animator?.(id, shape, { ...shape.attributes, opacity: originalOpacity }, { opacity: 0 }) || null, {
                    after: () => {
                        (0, element_2.updateStyle)(shape, { visibility: this.latestElementVisibilityMap.get(shape) });
                        afterUpdate();
                    },
                });
            }
        }
        const originalStyle = { ...shape.attributes };
        (0, element_2.updateStyle)(shape, style);
        return () => (0, animation_1.withAnimationCallbacks)(animator?.(id, shape, originalStyle) || null, {
            after: afterUpdate,
        });
    }
    getUpdateTasks(data, context) {
        const { nodes, edges, combos } = data;
        const iteration = [
            ['node', nodes],
            ['combo', combos],
            ['edge', edges],
        ];
        const { animation, stage } = context;
        const tasks = [];
        iteration.forEach(([elementType, elementData]) => {
            if (elementData.size === 0)
                return [];
            const animator = this.getAnimationExecutor(elementType, stage || 'update', animation);
            elementData.forEach((datum) => tasks.push(() => this.updateElement(elementType, datum, { ...context, animator })));
        });
        return tasks;
    }
    destroyElement(elementType, datum, context) {
        const { animator } = context;
        const id = (0, id_1.idOf)(datum);
        const element = this.elementMap[id];
        if (!element)
            return () => null;
        this.emit(new event_1.ElementLifeCycleEvent(constants_1.GraphEvent.BEFORE_ELEMENT_DESTROY, elementType, datum));
        return () => {
            const result = animator?.(id, element, { ...element.attributes }, { opacity: 0 }) || null;
            (0, animation_1.withAnimationCallbacks)(result, {
                after: () => {
                    this.clearElement(id);
                    element.destroy();
                    element.onDestroy();
                    this.emit(new event_1.ElementLifeCycleEvent(constants_1.GraphEvent.AFTER_ELEMENT_DESTROY, elementType, datum));
                },
            });
            return result;
        };
    }
    getDestroyTasks(data, context) {
        const { nodes, edges, combos } = data;
        const iteration = [
            ['combo', combos],
            ['edge', edges],
            ['node', nodes],
        ];
        const { animation, stage } = context;
        const tasks = [];
        iteration.forEach(([elementType, elementData]) => {
            if (elementData.size === 0)
                return [];
            const animator = this.getAnimationExecutor(elementType, stage || 'exit', animation);
            elementData.forEach((datum) => tasks.push(() => this.destroyElement(elementType, datum, { ...context, animator })));
        });
        // TODO 重新计算色板样式，如果是分组色板，则不需要重新计算
        return tasks;
    }
    clearElement(id) {
        delete this.paletteStyle[id];
        delete this.defaultStyle[id];
        delete this.stateStyle[id];
        delete this.elementMap[id];
        delete this.shapeTypeMap[id];
    }
    destroy() {
        Object.values(this.elementMap).forEach((element) => element.destroy());
        Object.values(this.container).forEach((container) => container.destroy());
        // @ts-expect-error force delete
        this.container = {};
        this.elementMap = {};
        this.shapeTypeMap = {};
        this.defaultStyle = {};
        this.stateStyle = {};
        this.paletteStyle = {};
        // @ts-expect-error force delete
        this.context = {};
        // @ts-expect-error force delete
        this.latestElementVisibilityMap = undefined;
    }
}
exports.ElementController = ElementController;
//# sourceMappingURL=element.js.map