"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Graph = void 0;
const event_emitter_1 = __importDefault(require("@antv/event-emitter"));
const util_1 = require("@antv/util");
const constants_1 = require("../constants");
const registry_1 = require("../registry");
const dom_1 = require("../utils/dom");
const event_1 = require("../utils/event");
const id_1 = require("../utils/id");
const point_1 = require("../utils/point");
const style_1 = require("../utils/style");
const vector_1 = require("../utils/vector");
const batch_1 = require("./batch");
const behavior_1 = require("./behavior");
const canvas_1 = require("./canvas");
const data_1 = require("./data");
const element_1 = require("./element");
const layout_1 = require("./layout");
const plugin_1 = require("./plugin");
const transform_1 = require("./transform");
const viewport_1 = require("./viewport");
class Graph extends event_emitter_1.default {
    options;
    static defaultOptions = {
        autoResize: false,
        width: 800,
        height: 600,
        theme: 'light',
        rotation: 0,
        zoom: 1,
        zoomRange: [0.01, 10],
    };
    destroyed = false;
    // @ts-expect-error will be initialized in createRuntime
    context = {
        model: new data_1.DataController(),
    };
    constructor(options) {
        super();
        this.options = Object.assign({}, Graph.defaultOptions, options);
        this.setOptions(this.options);
        this.context.graph = this;
        // Listening window.resize to autoResize.
        this.options.autoResize && window.addEventListener('resize', this.onResize);
    }
    /**
     * <zh/> 获取配置项
     *
     * <en/> Get options
     * @returns <zh/> 配置项 | <en/> options
     */
    getOptions() {
        return this.options;
    }
    /**
     * <zh/> 设置配置项
     *
     * <en/> Set options
     * @param options - <zh/> 配置项 | <en/> options
     * @description
     * <zh/> 要更新 devicePixelRatio、container 属性请销毁后重新创建实例
     *
     * <en/> To update devicePixelRatio and container properties, please destroy and recreate the instance
     */
    setOptions(options) {
        const { animation, background, behaviors, combo, container, data, edge, height, layout, node, padding, plugins, theme, transforms, width, zoom, zoomRange, } = options;
        if (background)
            this.setBackground(background);
        if (behaviors)
            this.setBehaviors(behaviors);
        if (combo)
            this.setCombo(combo);
        if (data)
            this.setData(data);
        if (edge)
            this.setEdge(edge);
        if (layout)
            this.setLayout(layout);
        if (node)
            this.setNode(node);
        if (theme)
            this.setTheme(theme);
        if (plugins)
            this.setPlugins(plugins);
        if (transforms)
            this.setTransforms(transforms);
        if ((0, util_1.isNumber)(width) || (0, util_1.isNumber)(height))
            this.setSize(width ?? this.options.width ?? 0, height ?? this.options.height ?? 0);
        if (zoomRange)
            this.options.zoomRange = zoomRange;
        if ((0, util_1.isNumber)(zoom))
            this.options.zoom = zoom;
        if ((0, util_1.isNumber)(padding))
            this.options.padding = padding;
    }
    setBackground(background) {
        this.options.background = background;
        this.context.canvas?.setBackground(background);
    }
    getBackground() {
        return this.options.background;
    }
    getSize() {
        if (this.context.canvas)
            return this.context.canvas.getSize();
        return [this.options.width || 0, this.options.height || 0];
    }
    setSize(width, height) {
        this.options.width = width;
        this.options.height = height;
    }
    setZoomRange(zoomRange) {
        this.options.zoomRange = zoomRange;
    }
    getZoomRange() {
        return this.options.zoomRange;
    }
    setNode(node) {
        this.options.node = node;
    }
    setEdge(edge) {
        this.options.edge = edge;
    }
    setCombo(combo) {
        this.options.combo = combo;
    }
    getTheme() {
        return this.options.theme;
    }
    setTheme(theme) {
        this.options.theme = (0, util_1.isFunction)(theme) ? theme(this.getTheme()) : theme;
        const { background } = (0, registry_1.getExtension)('theme', this.options.theme) || {};
        if (background && !this.options.background) {
            this.setBackground(background);
        }
    }
    setLayout(layout) {
        this.options.layout = (0, util_1.isFunction)(layout) ? layout(this.getLayout()) : layout;
    }
    getLayout() {
        return this.options.layout;
    }
    /**
     * <zh/> 设置交互
     *
     * <en/> Set behaviors
     * @param behaviors - <zh/> 交互配置 | <en/> behavior configuration
     */
    setBehaviors(behaviors) {
        this.options.behaviors = (0, util_1.isFunction)(behaviors) ? behaviors(this.getBehaviors()) : behaviors;
        this.context.behavior?.setBehaviors(this.options.behaviors);
    }
    /**
     * <zh/> 更新交互
     *
     * <en/> Update behavior
     * @param behavior - <zh/> 交互配置 | <en/> behavior configuration
     */
    updateBehavior(behavior) {
        this.setBehaviors((behaviors) => behaviors.map((_behavior) => {
            if ((0, util_1.isObject)(_behavior) && _behavior.key === behavior.key) {
                return { ..._behavior, ...behavior };
            }
            return _behavior;
        }));
    }
    /**
     * <zh/> 获取交互
     *
     * <en/> Get behaviors
     * @returns <zh/> 交互配置 | <en/> behavior configuration
     */
    getBehaviors() {
        return this.options.behaviors || [];
    }
    /**
     * <zh/> 设置插件
     *
     * <en/> Set plugins
     * @param plugins - <zh/> 插件配置 | <en/> plugin configuration
     */
    setPlugins(plugins) {
        this.options.plugins = (0, util_1.isFunction)(plugins) ? plugins(this.getPlugins()) : plugins;
        this.context.plugin?.setPlugins(this.options.plugins);
    }
    /**
     * <zh/> 更新插件
     *
     * <en/> Update plugin
     * @param plugin - <zh/> 插件配置 | <en/> plugin configuration
     */
    updatePlugin(plugin) {
        this.setPlugins((plugins) => plugins.map((_plugin) => {
            if ((0, util_1.isObject)(_plugin) && _plugin.key === plugin.key) {
                return { ..._plugin, ...plugin };
            }
            return _plugin;
        }));
    }
    /**
     * <zh/> 获取插件
     *
     * <en/> Get plugins
     * @returns <zh/> 插件配置 | <en/> plugin configuration
     */
    getPlugins() {
        return this.options.plugins || [];
    }
    /**
     * <zh/> 获取插件实例
     *
     * <en/> Get plugin instance
     * @param key - <zh/> 插件 key | <en/> plugin key
     * @returns <zh/> 插件实例 | <en/> plugin instance
     */
    getPluginInstance(key) {
        return this.context.plugin.getPluginInstance(key);
    }
    /**
     * <zh/> 设置数据转换器
     *
     * <en/> Set data transforms
     * @param transforms - <zh/> 数据转换配置 | <en/> data transform configuration
     */
    setTransforms(transforms) {
        this.options.transforms = (0, util_1.isFunction)(transforms) ? transforms(this.getTransforms()) : transforms;
        this.context.transform?.setTransforms(this.options.transforms);
    }
    /**
     * <zh/> 更新数据转换器
     *
     * <en/> Update data transform
     * @param transform - <zh/> 数据转换配置 | <en/> data transform configuration
     */
    updateTransform(transform) {
        this.setTransforms((transforms) => transforms.map((_transform) => {
            if ((0, util_1.isObject)(_transform) && _transform.key === transform.key) {
                return { ..._transform, ...transform };
            }
            return _transform;
        }));
    }
    /**
     * <zh/> 获取数据转换器
     *
     * <en/> Get data transforms
     * @returns <zh/> 数据转换器实例 | <en/> data transform instance
     */
    getTransforms() {
        return this.options.transforms || [];
    }
    getData() {
        return this.context.model.getData();
    }
    getNodeData(id) {
        if (id === undefined)
            return this.context.model.getNodeData();
        if (Array.isArray(id))
            return this.context.model.getNodeData(id);
        return this.context.model.getNodeData([id])?.[0];
    }
    getEdgeData(id) {
        if (id === undefined)
            return this.context.model.getEdgeData();
        if (Array.isArray(id))
            return this.context.model.getEdgeData(id);
        return this.context.model.getEdgeData([id])?.[0];
    }
    getComboData(id) {
        if (id === undefined)
            return this.context.model.getComboData();
        if (Array.isArray(id))
            return this.context.model.getComboData(id);
        return this.context.model.getComboData([id])?.[0];
    }
    setData(data) {
        this.context.model.setData((0, util_1.isFunction)(data) ? data(this.getData()) : data);
    }
    addData(data) {
        this.context.model.addData((0, util_1.isFunction)(data) ? data(this.getData()) : data);
    }
    addNodeData(data) {
        this.context.model.addNodeData((0, util_1.isFunction)(data) ? data(this.getNodeData()) : data);
    }
    addEdgeData(data) {
        this.context.model.addEdgeData((0, util_1.isFunction)(data) ? data(this.getEdgeData()) : data);
    }
    addComboData(data) {
        this.context.model.addComboData((0, util_1.isFunction)(data) ? data(this.getComboData()) : data);
    }
    updateData(data) {
        this.context.model.updateData((0, util_1.isFunction)(data) ? data(this.getData()) : data);
    }
    updateNodeData(data) {
        this.context.model.updateNodeData((0, util_1.isFunction)(data) ? data(this.getNodeData()) : data);
    }
    updateEdgeData(data) {
        this.context.model.updateEdgeData((0, util_1.isFunction)(data) ? data(this.getEdgeData()) : data);
    }
    updateComboData(data) {
        this.context.model.updateComboData((0, util_1.isFunction)(data) ? data(this.getComboData()) : data);
    }
    removeData(id) {
        this.context.model.removeData((0, util_1.isFunction)(id) ? id(this.getData()) : id);
    }
    removeNodeData(ids) {
        this.context.model.removeNodeData((0, util_1.isFunction)(ids) ? ids(this.getNodeData()) : ids);
    }
    removeEdgeData(ids) {
        this.context.model.removeEdgeData((0, util_1.isFunction)(ids) ? ids(this.getEdgeData()) : ids);
    }
    removeComboData(ids) {
        this.context.model.removeComboData((0, util_1.isFunction)(ids) ? ids(this.getComboData()) : ids);
    }
    getElementType(id) {
        return this.context.model.getElementType(id);
    }
    getRelatedEdgesData(id, direction = 'both') {
        return this.context.model.getRelatedEdgesData(id, direction);
    }
    getNeighborNodesData(id) {
        return this.context.model.getNeighborNodesData(id);
    }
    /**
     * <zh/> 获取节点或 combo 的祖先元素数据
     *
     * <en/> Get the ancestor element data of the node or combo
     * @param id - <zh/> 节点或 combo ID | <en/> node or combo ID
     * @param hierarchy - <zh/> 指定树图层级关系还是 combo 层级关系 | <en/> specify tree or combo hierarchy relationship
     * @returns <zh/> 祖先元素数据 | <en/> ancestor element data
     * @description
     * <zh/> 数组中的顺序是从父节点到祖先节点
     *
     * <en/> The order in the array is from the parent node to the ancestor node
     */
    getAncestorsData(id, hierarchy) {
        return this.context.model.getAncestorsData(id, hierarchy);
    }
    /**
     * <zh/> 获取节点或 combo 的父元素数据
     *
     * <en/> Get the parent element data of the node or combo
     * @param id - <zh/> 节点或 combo ID | <en/> node or combo ID
     * @param hierarchy - <zh/> 指定树图层级关系还是 combo 层级关系 | <en/> specify tree or combo hierarchy relationship
     * @returns <zh/> 父元素数据 | <en/> parent element data
     */
    getParentData(id, hierarchy) {
        return this.context.model.getParentData(id, hierarchy);
    }
    /**
     * <zh/> 获取节点或 combo 的子元素数据
     *
     * <en/> Get the child element data of the node or combo
     * @param id - <zh/> 节点或 combo ID | <en/> node or combo ID
     * @returns <zh/> 子元素数据 | <en/> child element data
     */
    getChildrenData(id) {
        return this.context.model.getChildrenData(id);
    }
    getElementDataByState(elementType, state) {
        return this.context.model.getElementDataByState(elementType, state);
    }
    async initCanvas() {
        if (this.context.canvas)
            return await this.context.canvas.init();
        const { container = 'container', width, height, renderer, background } = this.options;
        if (container instanceof canvas_1.Canvas) {
            this.context.canvas = container;
            container.setBackground(background);
            await container.init();
        }
        else {
            const $container = (0, util_1.isString)(container) ? document.getElementById(container) : container;
            this.emit(constants_1.GraphEvent.BEFORE_CANVAS_INIT, { container: $container, width, height });
            const canvas = new canvas_1.Canvas({
                container: $container,
                width,
                height,
                renderer,
            });
            this.context.canvas = canvas;
            await canvas.init();
            this.emit(constants_1.GraphEvent.AFTER_CANVAS_INIT, { canvas });
        }
    }
    createRuntime() {
        this.context.options = this.options;
        if (!this.context.batch)
            this.context.batch = new batch_1.BatchController(this.context);
        if (!this.context.plugin)
            this.context.plugin = new plugin_1.PluginController(this.context);
        if (!this.context.viewport)
            this.context.viewport = new viewport_1.ViewportController(this.context);
        if (!this.context.transform)
            this.context.transform = new transform_1.TransformController(this.context);
        if (!this.context.element)
            this.context.element = new element_1.ElementController(this.context);
        if (!this.context.layout)
            this.context.layout = new layout_1.LayoutController(this.context);
        if (!this.context.behavior)
            this.context.behavior = new behavior_1.BehaviorController(this.context);
    }
    async prepare() {
        if (this.destroyed)
            throw new Error('Graph has been destroyed');
        await this.initCanvas();
        this.createRuntime();
    }
    /**
     * <zh/> 执行渲染
     *
     * <en/> Render
     * @description
     * <zh/> 此过程会执行数据更新、绘制元素、执行布局
     *
     * <en/> This process will execute data update, element rendering, and layout execution
     */
    async render() {
        (0, event_1.emit)(this, new event_1.GraphLifeCycleEvent(constants_1.GraphEvent.BEFORE_RENDER));
        await this.prepare();
        await Promise.all([this.context.element?.draw(), this.context.layout?.layout()]);
        await this.autoFit();
        (0, event_1.emit)(this, new event_1.GraphLifeCycleEvent(constants_1.GraphEvent.AFTER_RENDER));
    }
    /**
     * <zh/> 绘制元素
     *
     * <en/> Draw elements
     * @returns <zh/> 渲染结果 | <en/> draw result
     */
    async draw() {
        await this.prepare();
        return this.context.element.draw();
    }
    layout() {
        return this.context.layout.layout();
    }
    stopLayout() {
        return this.context.layout.stopLayout();
    }
    /**
     * <zh/> 清空画布元素
     *
     * <en/> Clear canvas elements
     */
    async clear() {
        this.context.model.setData({});
        await this.draw();
    }
    destroy() {
        const { layout, element, model, canvas, behavior, plugin } = this.context;
        plugin?.destroy();
        behavior?.destroy();
        layout?.destroy();
        element?.destroy();
        model.destroy();
        canvas?.destroy();
        this.options = {};
        // @ts-expect-error force delete
        this.context = {};
        this.off();
        window.removeEventListener('resize', this.onResize);
        this.destroyed = true;
    }
    getCanvas() {
        return this.context.canvas;
    }
    resize(width, height) {
        const size = !width || !height ? (0, dom_1.sizeOf)(this.context.canvas.getContainer()) : [width, height];
        if ((0, util_1.isEqual)(size, this.getSize()))
            return;
        (0, event_1.emit)(this, new event_1.GraphLifeCycleEvent(constants_1.GraphEvent.BEFORE_SIZE_CHANGE, { size }));
        this.context.canvas.resize(...size);
        (0, event_1.emit)(this, new event_1.GraphLifeCycleEvent(constants_1.GraphEvent.AFTER_SIZE_CHANGE, { size }));
    }
    /**
     * <zh/> 将图缩放至合适大小并平移至视口中心
     *
     * <en/> Zoom the graph to fit the viewport and move it to the center of the viewport
     * @param options - <zh/> 适配配置 | <en/> fit options
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    async fitView(options, animation) {
        await this.context.viewport?.fitView(options, animation);
    }
    /**
     * <zh/> 将图平移至视口中心
     *
     * <en/> Move the graph to the center of the viewport
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    async fitCenter(animation) {
        await this.context.viewport?.fitCenter(animation);
    }
    async autoFit() {
        const { autoFit } = this.context.options;
        if (!autoFit)
            return;
        if ((0, util_1.isString)(autoFit)) {
            if (autoFit === 'view')
                await this.fitView();
            else if (autoFit === 'center')
                await this.fitCenter();
        }
        else {
            const { type, animation } = autoFit;
            if (type === 'view')
                await this.fitView(autoFit.options, animation);
            else if (type === 'center')
                await this.fitCenter(animation);
        }
    }
    async focusElement(id, animation) {
        await this.context.viewport?.focusElements(Array.isArray(id) ? id : [id], animation);
    }
    /**
     * <zh/> 基于当前缩放比例进行缩放
     *
     * <en/> Zoom based on the current zoom ratio
     * @param ratio - <zh/> 缩放比例 | <en/> zoom ratio
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     * @param origin - <zh/> 缩放中心(视口坐标) | <en/> zoom center(viewport coordinates)
     * @description
     * <zh/>
     * - ratio > 1 放大
     * - ratio < 1 缩小
     * <en/>
     * - ratio > 1 zoom in
     * - ratio < 1 zoom out
     */
    async zoomBy(ratio, animation, origin) {
        await this.context.viewport.transform({ mode: 'relative', scale: ratio, origin }, animation);
    }
    /**
     * <zh/> 缩放画布至指定比例
     *
     * <en/> Zoom the canvas to the specified ratio
     * @param zoom - <zh/> 指定缩放比例 | <en/> specified zoom ratio
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     * @param origin - <zh/> 缩放中心(视口坐标) | <en/> zoom center(viewport coordinates)
     * @description
     * <zh/>
     * - zoom = 1 默认大小
     * - zoom > 1 放大
     * - zoom < 1 缩小
     * <en/>
     * - zoom = 1 default size
     * - zoom > 1 zoom in
     * - zoom < 1 zoom out
     */
    async zoomTo(zoom, animation, origin) {
        this.context.viewport.transform({ mode: 'absolute', scale: zoom, origin }, animation);
    }
    /**
     * <zh/> 获取当前缩放比例
     *
     * <en/> Get the current zoom ratio
     * @returns <zh/> 缩放比例 | <en/> zoom ratio
     */
    getZoom() {
        return this.context.viewport.getZoom();
    }
    /**
     * <zh/> 基于当前旋转角度进行旋转
     *
     * <en/> Rotate based on the current rotation angle
     * @param angle - <zh/> 旋转角度 | <en/> rotation angle
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     * @param origin - <zh/> 旋转中心(视口坐标) | <en/> rotation center(viewport coordinates)
     */
    async rotateBy(angle, animation, origin) {
        await this.context.viewport.transform({ mode: 'relative', rotate: angle, origin }, animation);
    }
    /**
     * <zh/> 旋转画布至指定角度
     *
     * <en/> Rotate the canvas to the specified angle
     * @param angle - <zh/> 目标角度 | <en/> target angle
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     * @param origin - <zh/> 旋转中心(视口坐标) | <en/> rotation center(viewport coordinates)
     */
    async rotateTo(angle, animation, origin) {
        await this.context.viewport.transform({ mode: 'absolute', rotate: angle, origin }, animation);
    }
    /**
     * <zh/> 获取当前旋转角度
     *
     * <en/> Get the current rotation angle
     * @returns <zh/> 旋转角度 | <en/> rotation angle
     */
    getRotation() {
        return this.context.viewport.getRotation();
    }
    /**
     * <zh/> 将图平移指定距离
     *
     * <en/> Translate the graph by the specified distance
     * @param offset - <zh/> 偏移量 | <en/> offset
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    async translateBy(offset, animation) {
        await this.context.viewport.transform({ mode: 'relative', translate: offset }, animation);
    }
    /**
     * <zh/> 将图平移至指定位置
     *
     * <en/> Translate the graph to the specified position
     * @param position - <zh/> 指定位置 | <en/> specified position
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     * @description
     */
    async translateTo(position, animation) {
        await this.context.viewport.transform({ mode: 'absolute', translate: position }, animation);
    }
    /**
     * <zh/> 获取图的位置
     *
     * <en/> Get the position of the graph
     * @returns <zh/> 图的位置 | <en/> position of the graph
     * @description
     * <zh/> 默认状态下，图的位置为 [0, 0]
     *
     * <en/> By default, the position of the graph is [0, 0]
     */
    getPosition() {
        return (0, vector_1.subtract)([0, 0], this.getCanvasByViewport([0, 0]));
    }
    async translateElementBy(args1, args2, args3 = true) {
        const [config, animation] = (0, util_1.isObject)(args1)
            ? [args1, args2 ?? true]
            : [{ [args1]: args2 }, args3];
        Object.entries(config).forEach(([id, offset]) => this.context.model.translateNodeBy(id, offset));
        await this.context.element.draw({ animation });
    }
    async translateElementTo(args1, args2, args3 = true) {
        const [config, animation] = (0, util_1.isObject)(args1)
            ? [args1, args2 ?? true]
            : [{ [args1]: args2 }, args3];
        Object.entries(config).forEach(([id, position]) => this.context.model.translateNodeTo(id, position));
        await this.context.element.draw({ animation });
    }
    getElementPosition(id) {
        return this.context.model.getElementPosition(id);
    }
    getElementRenderStyle(id) {
        return (0, util_1.omit)(this.context.element.getElement(id).attributes, ['context']);
    }
    async setElementVisibility(args1, args2, args3 = true) {
        const [config, animation] = (0, util_1.isObject)(args1)
            ? [args1, args2 ?? true]
            : [{ [args1]: args2 }, args3];
        const dataToUpdate = { nodes: [], edges: [], combos: [] };
        Object.entries(config).forEach(([id, value]) => {
            const elementType = this.getElementType(id);
            dataToUpdate[`${elementType}s`].push({ id, style: { visibility: value } });
        });
        this.updateData(dataToUpdate);
        await this.context.element.draw({ animation, stage: 'visibility' });
    }
    /**
     * <zh/> 显示元素
     *
     * <en/> Show element
     * @param id - <zh/> 元素 ID | <en/> element ID
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    async showElement(id, animation) {
        const ids = Array.isArray(id) ? id : [id];
        await this.setElementVisibility(Object.fromEntries(ids.map((_id) => [_id, 'visible'])), animation);
    }
    /**
     * <zh/> 隐藏元素
     *
     * <en/> Hide element
     * @param id - <zh/> 元素 ID | <en/> element ID
     * @param animation - <zh/> 动画配置 | <en/> animation configuration
     */
    async hideElement(id, animation) {
        const ids = Array.isArray(id) ? id : [id];
        await this.setElementVisibility(Object.fromEntries(ids.map((_id) => [_id, 'hidden'])), animation);
    }
    /**
     * <zh/> 获取元素可见性
     *
     * <en/> Get element visibility
     * @param id - <zh/> 元素 ID | <en/> element ID
     * @returns <zh/> 元素可见性 | <en/> element visibility
     */
    getElementVisibility(id) {
        const element = this.context.element.getElement(id);
        return element?.style?.visibility ?? 'visible';
    }
    async setElementZIndex(args1, args2) {
        const dataToUpdate = { nodes: [], edges: [], combos: [] };
        const config = (0, util_1.isObject)(args1) ? args1 : { [args1]: args2 };
        Object.entries(config).forEach(([id, value]) => {
            const elementType = this.getElementType(id);
            dataToUpdate[`${elementType}s`].push({ id, style: { zIndex: value } });
        });
        const { model, element } = this.context;
        model.preventUpdateNodeLikeHierarchy(() => model.updateData(dataToUpdate));
        await element.draw({ animation: false });
    }
    /**
     * <zh/> 将元素置于最顶层
     *
     * <en/> Bring the element to the front
     * @param id - <zh/> 元素 ID | <en/> element ID
     */
    async frontElement(id) {
        const ids = Array.isArray(id) ? id : [id];
        const { model } = this.context;
        const config = {};
        ids.map((_id) => {
            const zIndex = model.getFrontZIndex(_id);
            const elementType = model.getElementType(_id);
            if (elementType === 'combo') {
                const ancestor = model.getAncestorsData(_id, constants_1.COMBO_KEY).at(-1) || this.getComboData(_id);
                const combos = [ancestor, ...model.getDescendantsData((0, id_1.idOf)(ancestor))].filter((datum) => model.isCombo((0, id_1.idOf)(datum)));
                const delta = zIndex - (0, style_1.zIndexOf)(ancestor);
                combos.forEach((combo) => {
                    config[(0, id_1.idOf)(combo)] = (0, style_1.zIndexOf)(combo) + delta;
                });
            }
            else
                config[_id] = zIndex;
        });
        await this.setElementZIndex(config);
    }
    /**
     * <zh/> 获取元素层级
     *
     * <en/> Get element z-index
     * @param id - <zh/> 元素 ID | <en/> element ID
     * @returns <zh/> 元素层级 | <en/> element z-index
     */
    getElementZIndex(id) {
        return (0, style_1.zIndexOf)(this.context.model.getElementsData([id])[0]);
    }
    async setElementState(args1, args2, args3 = true) {
        const [config, animation] = (0, util_1.isObject)(args1)
            ? [args1, args2 ?? true]
            : [{ [args1]: args2 }, args3];
        const parseState = (state) => {
            if (!state)
                return [];
            return Array.isArray(state) ? state : [state];
        };
        const dataToUpdate = { nodes: [], edges: [], combos: [] };
        Object.entries(config).forEach(([id, value]) => {
            const elementType = this.getElementType(id);
            dataToUpdate[`${elementType}s`].push({ id, style: { states: parseState(value) } });
        });
        this.updateData(dataToUpdate);
        await this.context.element.draw({ animation });
    }
    /**
     * <zh/> 获取元素状态
     *
     * <en/> Get element state
     * @param id - <zh/> 元素 ID | <en/> element ID
     * @returns <zh/> 元素状态 | <en/> element state
     */
    getElementState(id) {
        return this.context.model.getElementState(id);
    }
    getElementRenderBounds(id) {
        return this.context.element.getElement(id).getRenderBounds();
    }
    async collapse(id, animation = true) {
        this.setElementCollapsibility(id, true);
        await this.context.element.draw({ animation, stage: 'collapse' });
    }
    async expand(id, animation = true) {
        this.setElementCollapsibility(id, false);
        await this.context.element.draw({ animation, stage: 'expand' });
    }
    setElementCollapsibility(id, collapsed) {
        const elementType = this.getElementType(id);
        if (elementType === 'node')
            this.updateNodeData([{ id, style: { collapsed } }]);
        else if (elementType === 'combo')
            this.updateComboData([{ id, style: { collapsed } }]);
    }
    /**
     * <zh/> 导出画布内容为 DataURL
     *
     * <en/> Export canvas content as DataURL
     * @param options - <zh/> 导出配置 | <en/> export configuration
     * @param mode
     * <zh/> 导出模式
     *  - viewport: 导出视口内容
     *  - overall: 导出整个画布
     *
     * <en/> export mode
     *  - viewport: export the content of the viewport
     *  - overall: export the entire canvas
     * @returns <zh/> DataURL | <en/> DataURL
     */
    async toDataURL(options = {}) {
        return this.context.canvas.toDataURL(options);
    }
    getCanvasByViewport(point) {
        return (0, point_1.parsePoint)(this.context.canvas.viewport2Canvas((0, point_1.toPointObject)(point)));
    }
    getViewportByCanvas(point) {
        return (0, point_1.parsePoint)(this.context.canvas.canvas2Viewport((0, point_1.toPointObject)(point)));
    }
    getClientByCanvas(point) {
        const viewportPoint = this.context.canvas.canvas2Viewport((0, point_1.toPointObject)(point));
        return (0, point_1.parsePoint)(this.context.canvas.viewport2Canvas(viewportPoint));
    }
    getCanvasByClient(point) {
        const viewportPoint = this.context.canvas.client2Viewport((0, point_1.toPointObject)(point));
        return (0, point_1.parsePoint)(this.context.canvas.viewport2Canvas(viewportPoint));
    }
    /**
     * <zh/> 获取视口中心的画布坐标
     *
     * <en/> Get the canvas coordinates of the viewport center
     * @returns <zh/> 视口中心的画布坐标 | <en/> Canvas coordinates of the viewport center
     */
    getViewportCenter() {
        return this.context.viewport.getViewportCenter();
    }
    getCanvasCenter() {
        return this.context.viewport.getCanvasCenter();
    }
    onResize = (0, util_1.debounce)(() => {
        this.resize();
    }, 300);
    /**
     * <zh/> 监听事件
     *
     * <en/> Listen to events
     * @param eventName - <zh/> 事件名称 | <en/> event name
     * @param callback - <zh/> 回调函数 | <en/> callback function
     * @returns <zh/> Graph 实例 | <en/> Graph instance
     */
    on(eventName, callback) {
        return super.on(eventName, callback);
    }
    /**
     * <zh/> 一次性监听事件
     *
     * <en/> Listen to events once
     * @param eventName - <zh/> 事件名称 | <en/> event name
     * @param callback - <zh/> 回调函数 | <en/> callback function
     * @returns <zh/> Graph 实例 | <en/> Graph instance
     */
    once(eventName, callback) {
        return super.once(eventName, callback);
    }
}
exports.Graph = Graph;
//# sourceMappingURL=graph.js.map